package gov.va.med.pharmacy.peps.common.object;

import java.util.Arrays;
import java.util.List;

/**
 * The VoListEnum is used in conjunction with the StdJavaListEnum to create a white list of safe classes used by
 * SafeObjectInputStream for safe deserialization of objects. VoListEnum contains all the classes assignable by ValueObject,
 * Difference, AbstractValidator, and the classes for fields declared by those classes.
 * 
 * As ValueObject classes are added to the code, the classes associated with the the new ValueObject class need to be added to
 * the safe classes list. The VoListEnumTest is used to walk through all of the classes within the PS_PPS_common project to
 * determine which classes need to be added to the enumerations as a safe class. For classes indicated as missing from the safe
 * list by VoListEnumTest, Pharmacy classes should be added to this VoListEnum. Other Java classes should be added to the
 * StdJavaListEnum.
 *
 */
public enum VoListEnum {

    /** The volist. */
    VOLIST;

    private static Class<?>[] voList = {
        gov.va.med.pharmacy.peps.common.vo.ManagedEntityVo.class,
        gov.va.med.pharmacy.peps.common.vo.NationalProductVo.class,
        gov.va.med.pharmacy.peps.common.vo.NdcVo.class,
        gov.va.med.pharmacy.peps.common.vo.OrderableItemVo.class,
        gov.va.med.pharmacy.peps.common.vo.ActiveIngredientVo.class,
        gov.va.med.pharmacy.peps.common.vo.IngredientVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugUnitVo.class,
        gov.va.med.pharmacy.peps.common.vo.AdministrationScheduleAssocVo.class,
        gov.va.med.pharmacy.peps.common.vo.AdministrationScheduleVo.class,
        gov.va.med.pharmacy.peps.common.vo.AdministrationScheduleTypeVo.class,
        gov.va.med.pharmacy.peps.common.vo.AtcCanisterVo.class,
        gov.va.med.pharmacy.peps.common.vo.WardGroupForAtcVo.class,
        gov.va.med.pharmacy.peps.common.vo.CategoriesVo.class,
        gov.va.med.pharmacy.peps.common.vo.ColorVo.class,
        gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo.class,
        gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo.class,
        gov.va.med.pharmacy.peps.common.vo.CopayTierVo.class,
        gov.va.med.pharmacy.peps.common.vo.ProductVo.class,
        gov.va.med.pharmacy.peps.common.vo.CsFedScheduleVo.class,
        gov.va.med.pharmacy.peps.common.vo.datafield.DataField.class,
        gov.va.med.pharmacy.peps.common.vo.FieldKey.class,
        gov.va.med.pharmacy.peps.common.vo.datafield.DataFields.class,
        gov.va.med.pharmacy.peps.common.vo.datafield.GroupDataField.class,
        gov.va.med.pharmacy.peps.common.vo.datafield.GroupListDataField.class,
        gov.va.med.pharmacy.peps.common.vo.datafield.ListDataField.class,
        gov.va.med.pharmacy.peps.common.vo.datafield.MultitextDataField.class,
        gov.va.med.pharmacy.peps.common.vo.DifUpdateVo.class,
        gov.va.med.pharmacy.peps.common.vo.SiteUpdateScheduleVo.class,
        gov.va.med.pharmacy.peps.common.vo.DispenseUnitPerDoseVo.class,
        gov.va.med.pharmacy.peps.common.vo.DispenseUnitVo.class,
        gov.va.med.pharmacy.peps.common.vo.DosageFormNounVo.class,
        gov.va.med.pharmacy.peps.common.vo.DosageFormUnitVo.class,
        gov.va.med.pharmacy.peps.common.vo.DosageFormVo.class,
        gov.va.med.pharmacy.peps.common.vo.DoseUnitSynonymVo.class,
        gov.va.med.pharmacy.peps.common.vo.DoseUnitVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugClassGroupVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugClassVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugClassificationTypeVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugClassRowType.class,
        gov.va.med.pharmacy.peps.common.vo.DrugDataVendorVersionVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugDrugInteractionVo.class,
        gov.va.med.pharmacy.peps.common.vo.PrimaryIngredientVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugSeverityVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugMonographGcnseqnoAssocsVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugMonographVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugTextSynonymVo.class,
        gov.va.med.pharmacy.peps.common.vo.DrugTextVo.class,
        gov.va.med.pharmacy.peps.common.vo.EditSiteListVo.class,
        gov.va.med.pharmacy.peps.common.vo.EplVistaUpdateVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbAddVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbAutoAddVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbAutoUpdateVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbDosageFormVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbDrugClassVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbDrugIngredientVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbDrugUnitVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbGcnseqnoPemVVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbGcnseqnoPlblwVVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbGenericNameVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbMonographPemVVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbNdcVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbPlblwarningsVVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbProductVo.class,
        gov.va.med.pharmacy.peps.common.vo.FDBSearchOptionResultVo.class,
        gov.va.med.pharmacy.peps.common.vo.SingleMultiSourceProductVo.class,
        gov.va.med.pharmacy.peps.common.vo.FDBSearchOptionVo.class,
        gov.va.med.pharmacy.peps.common.vo.FdbUpdateVo.class,
        gov.va.med.pharmacy.peps.common.vo.FieldAuthorizationVo.class,
        gov.va.med.pharmacy.peps.common.vo.FileBasedSiteVo.class,
        gov.va.med.pharmacy.peps.common.vo.FormularyAlternativeVo.class,
        gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorVo.class,
        gov.va.med.pharmacy.peps.common.vo.GenericNameVo.class,
        gov.va.med.pharmacy.peps.common.vo.HospitalLocationSelectionVo.class,
        gov.va.med.pharmacy.peps.common.vo.HospitalLocationVo.class,
        gov.va.med.pharmacy.peps.common.vo.IfcapItemNumberVo.class,
        gov.va.med.pharmacy.peps.common.vo.IntendedUseVo.class,
        gov.va.med.pharmacy.peps.common.vo.InterfaceCounterVo.class,
        gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo.class,
        gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo.class,
        gov.va.med.pharmacy.peps.common.vo.LabDisplayAdministrationVo.class,
        gov.va.med.pharmacy.peps.common.vo.LabDisplayFinishingAnOrderVo.class,
        gov.va.med.pharmacy.peps.common.vo.LabDisplayOrderEntryVo.class,
        gov.va.med.pharmacy.peps.common.vo.LabTestMonitorVo.class,
        gov.va.med.pharmacy.peps.common.vo.LocalConsoleInfoVo.class,
        gov.va.med.pharmacy.peps.common.vo.LocalMedicationRouteVo.class,
        gov.va.med.pharmacy.peps.common.vo.StandardMedRouteVo.class,
        gov.va.med.pharmacy.peps.common.vo.LocalPossibleDosagesVo.class,
        gov.va.med.pharmacy.peps.common.vo.ManagedDataVo.class,
        gov.va.med.pharmacy.peps.common.vo.ManagedItemVo.class,
        gov.va.med.pharmacy.peps.common.vo.validator.Errors.class,
        gov.va.med.pharmacy.peps.common.vo.ManufacturerVo.class,
        gov.va.med.pharmacy.peps.common.vo.MedicationInstructionVo.class,
        gov.va.med.pharmacy.peps.common.vo.MergeDifferenceVo.class,
        gov.va.med.pharmacy.peps.common.vo.MergeVo.class,
        gov.va.med.pharmacy.peps.common.vo.MigrationCopayTierVo.class,
        gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo.class,
        gov.va.med.pharmacy.peps.common.vo.diff.Difference.class,
        gov.va.med.pharmacy.peps.common.vo.UserVo.class,
        gov.va.med.pharmacy.peps.common.vo.ModificationSummaryVo.class,
        gov.va.med.pharmacy.peps.common.vo.NationalPossibleDosagesVo.class,
        gov.va.med.pharmacy.peps.common.vo.OrderUnitVo.class,
        gov.va.med.pharmacy.peps.common.vo.NationalSettingVo.class,
        gov.va.med.pharmacy.peps.common.vo.NdcByOutpatientSiteNdcVo.class,
        gov.va.med.pharmacy.peps.common.vo.OutpatientSiteVo.class,
        gov.va.med.pharmacy.peps.common.vo.PackageTypeVo.class,
        gov.va.med.pharmacy.peps.common.vo.PackageSizeVo.class,
        gov.va.med.pharmacy.peps.common.vo.ShapeVo.class,
        gov.va.med.pharmacy.peps.common.vo.OtcRxVo.class,
        gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility.class,
        gov.va.med.pharmacy.peps.common.vo.NdfSynchQueueVo.class,
        gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo.class,
        gov.va.med.pharmacy.peps.common.vo.StatusVo.class,
        gov.va.med.pharmacy.peps.common.vo.NotificationSummaryVo.class,
        gov.va.med.pharmacy.peps.common.vo.NotificationTypeVo.class,
        gov.va.med.pharmacy.peps.common.vo.NotificationVo.class,
        gov.va.med.pharmacy.peps.common.vo.OiRouteVo.class,
        gov.va.med.pharmacy.peps.common.vo.OiScheduleTypeVo.class,
        gov.va.med.pharmacy.peps.common.vo.PackageUsageVo.class,
        gov.va.med.pharmacy.peps.common.vo.PartialSaveVo.class,
        gov.va.med.pharmacy.peps.common.vo.PatientMedicationInstructionVo.class,
        gov.va.med.pharmacy.peps.common.vo.PharmacySystemVo.class,
        gov.va.med.pharmacy.peps.common.vo.PossibleDosagesPackageVo.class,
        gov.va.med.pharmacy.peps.common.vo.PossibleIntendedUseVo.class,
        gov.va.med.pharmacy.peps.common.vo.PreferenceGroupVo.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.Column.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.FlowLinkColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.InputColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.LinkColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.LocalizableLinkColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.LocalizableResumeFlowLinkColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.PopupLinkColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.PrintTemplateVo.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.RedirectFlowLinkColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.ResumeFlowLinkColumn.class,
        gov.va.med.pharmacy.peps.common.vo.printtemplate.TextColumn.class,
        gov.va.med.pharmacy.peps.common.vo.ProcessedItemVo.class,
        gov.va.med.pharmacy.peps.common.vo.ProcessedRequestVo.class,
        gov.va.med.pharmacy.peps.common.vo.RequestVo.class,
        gov.va.med.pharmacy.peps.common.vo.SpecimenTypeVo.class,
        gov.va.med.pharmacy.peps.common.vo.RematchVo.class,
        gov.va.med.pharmacy.peps.common.vo.ReportDrugClassVo.class,
        gov.va.med.pharmacy.peps.common.vo.ReportFdbCopayChangeVo.class,
        gov.va.med.pharmacy.peps.common.vo.ReportProductVo.class,
        gov.va.med.pharmacy.peps.common.vo.ReportVo.class,
        gov.va.med.pharmacy.peps.common.vo.ReportVuidVo.class,
        gov.va.med.pharmacy.peps.common.vo.RequestMessageVo.class,
        gov.va.med.pharmacy.peps.common.vo.RequestSummaryVo.class,
        gov.va.med.pharmacy.peps.common.vo.RxConsultVo.class,
        gov.va.med.pharmacy.peps.common.vo.SearchCriteriaVo.class,
        gov.va.med.pharmacy.peps.common.vo.SearchFieldVo.class,
        gov.va.med.pharmacy.peps.common.vo.SearchNotificationsCriteriaVo.class,
        gov.va.med.pharmacy.peps.common.vo.SearchRequestCriteriaVo.class,
        gov.va.med.pharmacy.peps.common.vo.SearchTemplateMessageVo.class,
        gov.va.med.pharmacy.peps.common.vo.SearchTemplateVo.class,
        gov.va.med.pharmacy.peps.common.vo.SearchTermVo.class,
        gov.va.med.pharmacy.peps.common.vo.SessionPreferenceVo.class,
        gov.va.med.pharmacy.peps.common.vo.SiteConfigVo.class,
        gov.va.med.pharmacy.peps.common.vo.SpecialHandlingVo.class,
        gov.va.med.pharmacy.peps.common.vo.SynonymVo.class,
        gov.va.med.pharmacy.peps.common.vo.TradeNameVo.class,
        gov.va.med.pharmacy.peps.common.vo.UnitDoseRouteVo.class,
        gov.va.med.pharmacy.peps.common.vo.UnmatchRematchVo.class,
        gov.va.med.pharmacy.peps.common.vo.ValueObject.class,
        gov.va.med.pharmacy.peps.common.vo.VistaSiteVo.class,
        gov.va.med.pharmacy.peps.common.vo.VitalsDisplayAdministrationVo.class,
        gov.va.med.pharmacy.peps.common.vo.VitalsDisplayFinishAnOrderVo.class,
        gov.va.med.pharmacy.peps.common.vo.VitalsDisplayOrderEntryVo.class,
        gov.va.med.pharmacy.peps.common.vo.VuidStatusHistoryVo.class,
        gov.va.med.pharmacy.peps.common.vo.WardMultipleVo.class,
        gov.va.med.pharmacy.peps.common.vo.WardSelectionVo.class,
        gov.va.med.pharmacy.peps.common.vo.ManagedItemVo.ManagedItemComparator.class,
        gov.va.med.pharmacy.peps.common.vo.diff.GroupDifference.class,
        gov.va.med.pharmacy.peps.common.vo.diff.MergeDifference.class,
        gov.va.med.pharmacy.peps.common.vo.validator.AbstractValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.ActiveIngredientValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.AdministrationScheduleValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.AdminSchedulesValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.AtcCanistersValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.CopayTierValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.CsFedScheduleValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.group.DispenseQuantityLimitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.list.ApplicationPackageUseValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.list.DeaScheduleValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.list.DispenseUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.list.FormularyValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.list.NcpdpDispenseUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.list.OrderableItemSynonymValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.list.ProductTypeValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.ArWsAmisConversionNumberValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.CurrentInventoryValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.DawCodeValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.DaysSupplyLimitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.DispenseDaysSupplyLimitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.IenValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.MonitorMaxDaysValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.NdcPricePerDispenseUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.NdcPricePerOrderUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.NormalAmountToOrderValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.NumberOfDosesValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.ProductDispenseUnitPerOrderUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.ProductPricePerDispenseUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.ProductPricePerOrderUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.ReorderLevelValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.ServiceCodeValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.numeric.UnitPriceValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.CommonStringLength0To11Validator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.CommonStringLength0To12Validator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.CommonStringLength0To200Validator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.CommonStringLength0To40Validator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.CommonStringLength0To50Validator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.CommonStringLength1To120Validator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.CommonStringLength3To8Validator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.DayNdOrDoseNlLimitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.FsnValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.InpatientPharmacyLocationValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.LocalSpecialHandlingValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.MessageValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.NcpdpQuantityMultiplierValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.PreviousNdcValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.ProductNumberValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.QuantityDispenseMessageValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.UnitDoseScheduleValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.UpcUpnValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.VAPrintNameValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.datafield.string.WarningLabelValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DispenseUnitPerDoseValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DispenseUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DosageFormNounValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DosageFormUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DosageFormValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DoseUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DrugClassificationValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DrugClassValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DrugDrugInteractionValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DrugTextValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.DrugUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.EffectiveDateTimeTypeValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.FdbSearchValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.AdministrationTimesValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.CmopIdValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.FormularyAlternativeValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.FrequencyValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.GCNSequenceNumberValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.LocalPrintNameValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.MaximumDosePerDayValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.MigratedDosageFormNameValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.NationalFormularyNameValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.NdcFieldValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.OrderableItemTextValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.ProductStrengthValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.ProductTextValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.RxNormLanguageValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.SourceValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.StrengthValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.TallManLetteringValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.VAProductNameValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.VendorStockNumberValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.VendorValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.fields.VuidValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.FormularyDesignatorValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.GenericNameValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.HospitalLocationValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.IngredientValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.LocalMedicationRouteValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.LocalPossibleDosagesValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.ManufacturerValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.MedicationInstructionValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.MedInstructionWardValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.NdcByOutpatientSiteNdcValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.NdcValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.OiRouteValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.OrderableItemValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.OrderUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.OtcRxValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.PackageSizeValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.PackageTypeValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.PartialSaveValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.PharmacySystemValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.PossibleDosagesValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.ProductCopayTierValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.ProductValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.RxConsultValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.SearchCriteriaValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.SearchRequestCriteriaValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.SearchTemplateValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.SingleMultiSourceProductValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.SpecialHandlingForProductValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.SpecialHandlingValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.StandardMedicationRouteValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.SynonymValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.UpdateFileValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.UserValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.VADispenseUnitValidator.class,
        gov.va.med.pharmacy.peps.common.vo.validator.WardValidator.class
    };

    /**
     * Provide the enumeration as a list of classes. 
     * @return list of classes 
     */
    public static List<Class<?>> getVoList() {
        return Arrays.asList(voList);
    };
}
