package gov.va.med.pharmacy.peps.common.utility;

import java.io.File;

/**
 * <p>Provides general system configuration properties.</p>
 * 
 * <p>Hopefully this interface will eventually replace direct calls to {@link PropertyUtility} and maybe even some of
 * the NationalSettings classes.</p>
 * 
 * @since 3.0
 */
public interface PpsConfiguration {

  /**
   * getConnectionSpecName
   * @return connectionSpecName
   */
  String getConnectionSpecName();

  /**
   * getRpcTimeout
   * @return rpcTimeout
   */
  int getRpcTimeout();

  /**
   * getTimeout    
   * @return : timeout
   */
  int getTimeout();

  /**
   * getNdfProxyUserDUZ
   * @return ndfProxyUserDUZ
   */
  int getNdfProxyUserDUZ();

  /**
   * getDivision
   * @return division
   */
  int getDivision();

  /**
   * get the fdbImageLocation
   * 
   * @return the fdbImageLocation
   */
  String getFdbImageLocation();

  /**
   * Returns the STS WebService URL.
   * 
   * @return String
   */
  String getStsWebServiceURL();
  
  /**
   * Returns the directory where the system should store files that need to persist across restarts, e.g. the home page
   * image.
   * @return never null
   */
  File getPersistentDirectory();

    /**
     * Returns the ISAAC coding concept URL.
     * @return String
     */
    String getIsaacCodingConceptURL();

    /**
     * Returns the ISAAC coding component URL.
     * @return String
     */
    String getIsaacCodingComponentURL();

    /**
     * Returns the ISAAC coding descriptions URL.
     * @return String
     */    
    String getIsaacCodingDescriptionsURL();
}
