/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo;


import java.util.ArrayList;
import java.util.List;


/**
 * 
 * Enumeration of categories, which apply to Orderable Item, Product, and NDC.
 *
 */
public enum Category {

    /**
     * MEDICATION.
     */
    MEDICATION,

    /**
     * INVESTIGATIONAL.
     */
    INVESTIGATIONAL,

    /**
     * COMPOUND.
     */
    COMPOUND,

    /**
     * SUPPLY.
     */
    SUPPLY;

    /**
     * Get a list of all categories.
     * 
     * @return list of categories
     */
    public static List<Category> getAll() {

        ArrayList<Category> ret = new ArrayList<Category>();

        for (Category type : values()) {
            ret.add(type);
        }

        return ret;
    }
    
    /**
     * Compares if the two specified lists of Category objects are the same, ignoring order.
     *
     * @param list1 the list1
     * @param list2 the list2
     * @return true, if successful
     */
    public static boolean areListsTheSameIgnoreOrder(List<Category> list1, List<Category> list2){
      if (list1 == null && list2 == null){
        return true;        
      }
      
      if (list1 == null || list2 == null){
        return false;
      }
      
      return (list1.containsAll(list2) && list2.containsAll(list1));      
    }
}
