/**
 * Source file created in 2011 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo;


import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;

/**
 * Types available to a Copay Tier Association.
 */
public class CopayTierAssocVo extends ManagedDataVo implements NdfSyncable {

    private static final long serialVersionUID = 1L;

    /**
     * The primary key field for the product copay tier record.
     */
    private Long eplId;    
    
    /**
     * The date this product copay tier was effective.
     */
    private Date effectiveDate;    
    
    /**
     * The last day this copay tier association was in effect.
     */
    private Date endDate;
    
    /**
     * The associated copay tier.
     */
    private CopayTierVo copayTier;
    
    /**
     * The associated product.
     */
    private ProductVo product;

    /**
     * the constructor.
     */
    public CopayTierAssocVo() {

    }

    /**
     * Gets the getEplId.
     * 
     * @return eplId
     */
    public final Long getEplId() {

        return eplId;
    }

    /**
     * Sets the setEplId.
     * @param eplId eplId
     * 
     */
    public final void setEplId(Long eplId) {

        this.eplId = eplId;
    }
    
    public final Date getEffectiveDate() {
        
        return effectiveDate;
    }
    
    public final void setEffectiveDate(Date effectiveDate) {
    
        this.effectiveDate = effectiveDate;
    }
    
    public final Date getEndDate() {
    
        return endDate;
    }
    
    public final void setEndDate(Date endDate) {
    
        this.endDate = endDate;
    }
    
    public final CopayTierVo getCopayTier() {
        
        return copayTier;
    }

    
    public final void setCopayTier(CopayTierVo copayTier) {
    
        this.copayTier = copayTier;
    }

    public final void setProduct(ProductVo product) {
        this.product = product;
    }
    
    public final ProductVo getProduct() {
        return product;
    }
    
    /**
     * returns the domain group of the managed Data.
     * 
     * @return domainGroup
     */
    public final DomainGroup getDomainGroup() {
        //return DomainGroup.DRUG_CLASS_CODE;
        return null;
    }
    
    /**
     * Returns true or false to indicate whether this copay tier 
     * association is currently in effect.
     * 
     * @return true if and only if the effective date is before or
     * equal to the current date and the end date is blank, equal to, or
     * after the current date; otherwise, return false.
     */
    public final boolean isActive() {
        Date currentDate = new Date();
        return (effectiveDate.compareTo(currentDate) <= 0)
            && (endDate == null || endDate.compareTo(currentDate) >= 0);
    }
    
    /**
     * Returns true or false to indicate whether this copay tier association has an end date that is in the past.
     * 
     * @return true if and only if the end date is before the current date; otherwise, return false.
     */
    public final boolean isPast() {

        Date currentDate = new Date();
        return (endDate != null && endDate.compareTo(currentDate) < 0);
    }

    /**
     * Returns true or false to indicate whether this copay tier association has a future effective date.
     * 
     * @return true if and only if the effective date is after the current date; otherwise, return false.
     */
    public final boolean isFuture() {

        Date currentDate = new Date();
        return (effectiveDate.compareTo(currentDate) > 0);
    }
    
    /**
     * Returns true if this DrugClassVo is a local only domain.
     * 
     * @return false always
     */
    public final boolean isLocalOnlyData() {
        return false;
    }
    
    /**
     * Returns true if the domain is an NDF domain for the DrugClassVo.
     * 
     * @return boolean
     */
    public final boolean isNdf() {
        return true;
    }
    
    /**
     * Returns true if the domain is standardized for the CopayTierVo.
     * 
     * @return boolean
     */
    public final boolean isStandardized() {
        return false;
    }

    /**
     * Returns the dates in readable formats.
     * @see gov.va.med.pharmacy.peps.common.vo.ManagedDataVo#toShortString()
     * 
     * @return "Start Date: getProductCopayTierEffectiveDate.toString(), Stop Date: getProductCopayTierEndDate().toString()"
     */
    @Override
    public final String toShortString() {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        List<String> values = new ArrayList<String>();
        
        if(copayTier != null) {
            values.add(FieldKey.COPAY_TIER.getLocalizedAbbreviation(Locale.getDefault()) + ": " + copayTier.value);
        }

        if (getEffectiveDate() != null) {            
            values.add(FieldKey.PRODUCT_COPAY_TIER_EFFECTIVE_DATE.getLocalizedAbbreviation(Locale.getDefault()) + ": "
                + formatter.format(getEffectiveDate()));
        }

        if (getEndDate() != null) {
            values.add(FieldKey.PRODUCT_COPAY_TIER_END_DATE.getLocalizedAbbreviation(Locale.getDefault()) + ": "
                + formatter.format(getEndDate()));
        }

        String result = String.join("; ", values);
        return result;
    }

    /**
     * Finds active CopayTierAssocVo in the specified list.
     * In Java 8, this could be written: 
     * list.stream().filter(copayTierAssoc -> copayTierAssoc.isActive()).collect(Collectors.toList());
     *
     * @param list the list
     * @return a list containing just the active CopayTierAssocVo
     */
    public static final List<CopayTierAssocVo> filterActive(List<CopayTierAssocVo> list){
        List<CopayTierAssocVo> filteredList = new ArrayList<CopayTierAssocVo>();
        for (CopayTierAssocVo copayTierAssocVo : list) {
            if (copayTierAssocVo.isActive()){
                filteredList.add(copayTierAssocVo);
            }
        }
        return filteredList;
    }
    
    /**
     * Finds past CopayTierAssocVo in the specified list.
     * In Java 8, this could be written: 
     * list.stream().filter(copayTierAssoc -> copayTierAssoc.isPast()).collect(Collectors.toList());

     * @param list the list
     * @return a list containing just the past CopayTierAssocVo
     */
    public static final List<CopayTierAssocVo> filterPast(List<CopayTierAssocVo> list){
        List<CopayTierAssocVo> filteredList = new ArrayList<CopayTierAssocVo>();
        for (CopayTierAssocVo copayTierAssocVo : list) {
            if (copayTierAssocVo.isPast()){
                filteredList.add(copayTierAssocVo);
            }
        }
        return filteredList;
    }

    /**
     * Finds future CopayTierAssocVo in the specified list.
     * In Java 8, this could be written: 
     * list.stream().filter(copayTierAssoc -> copayTierAssoc.isFuture()).collect(Collectors.toList());

     * @param list the list
     * @return a list containing just the future CopayTierAssocVo
     */
    public static final List<CopayTierAssocVo> filterFuture(List<CopayTierAssocVo> list){
        List<CopayTierAssocVo> filteredList = new ArrayList<CopayTierAssocVo>();
        for (CopayTierAssocVo copayTierAssocVo : list) {
            if (copayTierAssocVo.isFuture()){
                filteredList.add(copayTierAssocVo);
            }
        }
        return filteredList;
    }

    /**
     * Finds active and future CopayTierAssocVo in the specified list.
     * In Java 8, this could be written: 
     * list.stream().filter(copayTierAssoc -> ((copayTierAssocVo.isActive() 
     *      || copayTierAssocVo.isFuture())).collect(Collectors.toList());
     * @param list the list
     * @return a list containing just the future CopayTierAssocVo
     */
    public static final List<CopayTierAssocVo> filterActiveAndFuture(List<CopayTierAssocVo> list){
        List<CopayTierAssocVo> filteredList = new ArrayList<CopayTierAssocVo>();
        for (CopayTierAssocVo copayTierAssocVo : list) {
            if (copayTierAssocVo.isActive() || copayTierAssocVo.isFuture()){
                filteredList.add(copayTierAssocVo);
            }
        }
        return filteredList;
    }

    @Override
    public List<SyncProcessorElement> asSyncProcessorElements(
        String vistaIen,
        int pseudoSubfieldIen, 
        String userValue) {
        List<SyncProcessorElement> elements = new ArrayList<>();
        
        String VistaIenChild = vistaIen + "," + pseudoSubfieldIen;

        elements.add(
            new SyncProcessorElement(
                VistaProductFile.FILE_NUMBER,
                VistaIenChild, 
                this.getCopayTier().getValue(), 
                userValue,
                VistaProductFile.COPAY_TIER_FIELD_NUMBER,
                SyncProcessorElement.IS_MULTIPLE_FIELD));

        elements.add(
            new SyncProcessorElement(
                VistaProductFile.FILE_NUMBER,
                VistaIenChild, 
                MumpsConverter.convertDate(effectiveDate), 
                userValue,
                VistaProductFile.COPAY_TIER_START_DATE_FIELD_NUMBER,
                SyncProcessorElement.IS_MULTIPLE_FIELD));

        if (endDate != null) {
            elements.add(
                new SyncProcessorElement(
                    VistaProductFile.FILE_NUMBER,
                    VistaIenChild, 
                    MumpsConverter.convertDate(endDate), 
                    userValue,
                    VistaProductFile.COPAY_TIER_END_DATE_FIELD_NUMBER,
                    SyncProcessorElement.IS_MULTIPLE_FIELD));
        }
        
        return elements;
    }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }
        
}
