package gov.va.med.pharmacy.peps.common.vo;

/**
 * Represents the data in a single row on the Hazardous Waste import spreadsheet, along with any errors occurring during the
 * validation and processing of that data.
 */
public class HazardousWasteImportRow extends MigrationRow{
    
    /** The vuid. */
    private String vuid;
    
    /** The epl id. */
    private Long eplId;
    
    /** The cmop id. */
    private String cmopId;
    
    /** The ndf product ien. */
    private Long ndfProductIen;
    
    /** The va product name. */
    private String vaProductName;
    
    /** The generic name. */
    private String genericName; // GenericNameVo
    
    /** The schedule name. */
    private String scheduleName; // not used by migration, just in file for users to verify records
    
    /** The dosage form name. */
    private String dosageFormName; // not used by migration, just in file for users to verify records
    
    /** The haz2 dispose flag str. */
    private String haz2DisposeFlagStr;
    
    /** The haz2 handle flag str. */
    private String haz2HandleFlagStr;
    
    /** The primary epa. */
    private String primaryEPA;
    
    /** The waste sort code. */
    private String wasteSortCode;
    
    /** The dot shipping name. */
    private String dotShippingName;
    
    /**
     * Constructor using all fields except errors.  Defaults errors to an empty list.
     *
     * @param vuid the vuid
     * @param eplId the epl id
     * @param cmopId the cmop id
     * @param ndfProductIen the ndf product ien
     * @param vaProductName the va product name
     * @param genericName the generic name
     * @param scheduleName the schedule name
     * @param dosageFormName the dosage form name
     * @param haz2DisposeFlagStr the haz2 dispose flag str
     * @param haz2HandleFlagStr the haz2 handle flag str
     * @param primaryEPA the primary epa
     * @param wasteSortCode the waste sort code
     * @param dotShippingName the dot shipping name
     */
    public HazardousWasteImportRow(String vuid, Long eplId, String cmopId, Long ndfProductIen, String vaProductName,
        String genericName, String scheduleName, String dosageFormName, String haz2DisposeFlagStr, String haz2HandleFlagStr,
        String primaryEPA, String wasteSortCode, String dotShippingName) {
        super();
        this.vuid = vuid;
        this.eplId = eplId;
        this.cmopId = cmopId;
        this.ndfProductIen = ndfProductIen;
        this.vaProductName = vaProductName;
        this.genericName = genericName;
        this.scheduleName = scheduleName;
        this.dosageFormName = dosageFormName;
        this.haz2DisposeFlagStr = haz2DisposeFlagStr;
        this.haz2HandleFlagStr = haz2HandleFlagStr;
        this.primaryEPA = primaryEPA;
        this.wasteSortCode = wasteSortCode;
        this.dotShippingName = dotShippingName;
        
    }

    /**
     * Gets the vuid.
     *
     * @return the vuid
     */
    public final String getVuid() {
        return vuid;
    }

    /**
     * Gets the epl id.
     *
     * @return the eplId
     */
    public final Long getEplId() {
        return eplId;
    }

    /**
     * Gets the cmop id.
     *
     * @return the cmopId
     */
    public final String getCmopId() {
        return cmopId;
    }

    /**
     * Gets the ndf product ien.
     *
     * @return the ndfProductIen
     */
    public final Long getNdfProductIen() {
        return ndfProductIen;
    }

    /**
     * Gets the va product name.
     *
     * @return the vaProductName
     */
    public final String getVaProductName() {
        return vaProductName;
    }

    /**
     * Gets the generic name.
     *
     * @return the genericName
     */
    public final String getGenericName() {
        return genericName;
    }

    /**
     * Gets the schedule name.
     *
     * @return the scheduleName
     */
    public final String getScheduleName() {
        return scheduleName;
    }

    /**
     * Gets the dosage form name.
     *
     * @return the dosageFormName
     */
    public final String getDosageFormName() {
        return dosageFormName;
    }

    /**
     * Gets the haz2 dispose flag str.
     *
     * @return the haz2DisposeFlagStr
     */
    public final String getHaz2DisposeFlagStr() {
        return haz2DisposeFlagStr;
    }

    /**
     * Gets the haz2 handle flag str.
     *
     * @return the haz2HandleFlagStr
     */
    public final String getHaz2HandleFlagStr() {
        return haz2HandleFlagStr;
    }

    /**
     * Gets the primary epa.
     *
     * @return the primaryEPA
     */
    public final String getPrimaryEPA() {
        return primaryEPA;
    }

    /**
     * Gets the waste sort code.
     *
     * @return the wasteSortCode
     */
    public final String getWasteSortCode() {
        return wasteSortCode;
    }

    /**
     * Gets the dot shipping name.
     *
     * @return the dotShippingName
     */
    public final String getDotShippingName() {
        return dotShippingName;
    }
    
    
}
