package gov.va.med.pharmacy.peps.common.vo;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.Date;

/**
 * This class represents a single row of data on the FDB
 * Copay Change report.
 */
public class ReportFdbCopayChangeVo extends ValueObject {

    private static final long serialVersionUID = 1L;

    /**
     * The unique identifier of the PPS-N product.
     */
    private String id;
    
    /**
     * The VA Product Name.
     */
    private String vaProductName;
    
    /**
     * The orderable item name.
     */
    private String oiName;
    
    /**
     * The unique identifier of the parent orderable item.
     */
    private String parentId;
    
    /**
     * The GCN sequence number.
     */
    private String gcnSequenceNumber;
    
    /**
     * For inactive products, the date the product was
     * inactivated in PPS-N.
     */
    private Date inactivationDate;
    
    /**
     * The computed tier level of the product in the FDB
     * data.
     */
    private String fdbTierLevel;
    
    /**
     * The future tier level of the product in PPS-N.
     */
    private String futureTierLevel;
    
    /**
     * The federal legend code for the product in the FDB
     * data.
     */
    private String federalLegendCode;
    
    /**
     * The entity type.
     */
    private EntityType entityType;
    
    /**
     * The constructor.
     */
    public ReportFdbCopayChangeVo() {
        super();
        entityType = EntityType.PRODUCT;
    }
    
    /**
     * Get the VA product name.
     * 
     * @return the VA product name.
     */
    public final String getVaProductName() {
    
        return vaProductName;
    }
    
    /**
     * Set the VA product name.
     * 
     * @param vaProductName vaProductName
     */
    public final void setVaProductName(String vaProductName) {
    
        this.vaProductName = vaProductName;
    }
    
    /**
     * Alternate setter for Hibernate to populate the VA product name
     * via the fdbCopayChangeReport native query.
     * 
     * @param vaProductNameParam vaProductNameParam
     */
    public final void setVA_PRODUCT_NAME(String vaProductNameParam) {
        
        this.vaProductName = vaProductNameParam;
    }
    
    /**
     * Get the orderable item name.
     * 
     * @return the orderable item name.
     */
    public final String getOiName() {
    
        return oiName;
    }
    
    /**
     * Set the orderable item name.
     * 
     * @param oiName oiName
     */
    public final void setOiName(String oiName) {
    
        this.oiName = oiName;
    }
    
    /**
     * Alternate setter for Hibernate to populate the orderable item name
     * via the fdbCopayChangeReport native query.
     * 
     * @param oiNameParam oiNameParam
     */
    public final void setOI_NAME(String oiNameParam) {
        
        this.oiName = oiNameParam;
    }
    
    /**
     * Get the GCN sequence number.
     * 
     * @return the GCN sequence number.
     */
    public final String getGcnSequenceNumber() {
    
        return gcnSequenceNumber;
    }
    
    /**
     * Set the GCN sequence number.
     * 
     * @param gcnSequenceNumber gcnSequenceNumber
     */
    public final void setGcnSequenceNumber(String gcnSequenceNumber) {
    
        this.gcnSequenceNumber = gcnSequenceNumber;
    }
    
    /**
     * Alternate setter for Hibernate to populate the GCN sequence number
     * via the fdbCopayChangeReport native query.
     * 
     * @param gcnSequenceNumberParam gcnSequenceNumberParam
     */
    public final void setGCN_SEQNO(BigDecimal gcnSequenceNumberParam) {
        if(gcnSequenceNumberParam != null) {
            this.gcnSequenceNumber = gcnSequenceNumberParam.toString();
        }
        else {
            this.gcnSequenceNumber = null;
        }
    }
    
    /**
     * Get the inactivation date.
     * 
     * @return the inactivation date.
     */
    public final Date getInactivationDate() {
    
        return inactivationDate;
    }
    
    /**
     * Set the inactivation date.
     * 
     * @param inactivationDate inactivationDate
     */
    public final void setInactivationDate(Date inactivationDate) {
    
        this.inactivationDate = inactivationDate;
    }
    
    /**
     * Alternate setter for Hibernate to populate the inactivation date
     * via the fdbCopayChangeReport native query.
     * 
     * @param inactivationDateParam inactivationDateParam
     */
    public final void setINACTIVATION_DATE(Date inactivationDateParam) {
        
        this.inactivationDate = inactivationDateParam;
    }
    
    /**
     * Get the FDB tier level.
     * 
     * @return the FDB tier level.
     */
    public final String getFdbTierLevel() {
    
        return fdbTierLevel;
    }
    
    /**
     * Set the FDB tier level.
     * 
     * @param fdbTierLevel fdbTierLevel
     */
    public final void setFdbTierLevel(String fdbTierLevel) {
    
        this.fdbTierLevel = fdbTierLevel;
    }
    
    /**
     * Alternate setter for Hibernate to populate the FDB tier level
     * via the fdbCopayChangeReport native query.
     * 
     * @param fdbTierLevelParam fdbTierLevelParam
     */
    public final void setFDB_TIER(Character fdbTierLevelParam) {
        if(fdbTierLevelParam != null) {
            this.fdbTierLevel = Character.toString(fdbTierLevelParam);
        }
        else {
            this.fdbTierLevel = null;
        }
    }
    
    /**
     * Get the future tier level.
     * 
     * @return the future tier level.
     */
    public final String getFutureTierLevel() {
    
        return futureTierLevel;
    }
    
    /**
     * Set the future tier level.
     * 
     * @param futureTierLevel futureTierLevel
     */
    public final void setFutureTierLevel(String futureTierLevel) {
    
        this.futureTierLevel = futureTierLevel;
    }
    
    /**
     * Alternate setter for Hibernate to populate the future tier level
     * via the fdbCopayChangeReport native query.
     * 
     * @param futureTierLevelParam futureTierLevelParam
     */
    public final void setFUTURE_TIER(String futureTierLevelParam) {
        
        this.futureTierLevel = futureTierLevelParam;
    }
    
    /**
     * Get the federal legend code.
     * 
     * @return the federal legend code.
     */
    public final String getFederalLegendCode() {
    
        return federalLegendCode;
    }
    
    /**
     * Set the federal legend code.
     * 
     * @param federalLegendCode federalLegendCode
     */
    public final void setFederalLegendCode(String federalLegendCode) {
    
        this.federalLegendCode = federalLegendCode;
    }

    /**
     * Alternate setter for Hibernate to populate the federal legend code
     * via the fdbCopayChangeReport native query.
     * 
     * @param federalLegendCodeParam federalLegendCodeParam
     */
    public final void setFEDERAL_LEGEND_CODE(String federalLegendCodeParam) {
        
        this.federalLegendCode = federalLegendCodeParam;
    }
    
    /**
     * Get the id.
     * 
     * @return the id.
     */
    public final String getId() {

        return id;
    }

    /**
     * Set the id.
     * 
     * @param id id
     */
    public final void setId(String id) {

        this.id = id;
    }

    /**
     * Alternate setter for Hibernate to populate the id
     * via the fdbCopayChangeReport native query.
     * 
     * @param idParam idParam
     */
    public final void setID(BigDecimal idParam) {
        if(idParam != null) {
            this.id = idParam.toString();
        }
        else {
            this.id = null;
        }
    }
    
    /**
     * Get the entity type.
     * 
     * @return the entity type.
     */
    public final EntityType getEntityType() {
    
        return entityType;
    }

    /**
     * Get the parent id.
     * 
     * @return the parent id.
     */
    public final String getParentId() {

        return parentId;
    }

    /**
     * Set the parent id.
     * 
     * @param parentId parentId
     */
    public final void setParentId(String parentId) {

        this.parentId = parentId;
    }
    
    /**
     * Alternate setter for Hibernate to populate the parent id
     * via the fdbCopayChangeReport native query.
     * 
     * @param parentIdParam parentIdParam
     */
    public final void setPARENT_ID(BigDecimal parentIdParam) {
        if(parentIdParam != null) {
            this.parentId = parentIdParam.toString();
        }
        else {
            this.parentId = null;
        }
    }

    @Override
    public <T> T reflectFieldValue(Field field) throws IllegalArgumentException, IllegalAccessException {
        return (T) field.get(this);
    }

    @Override
    public void setAttributeByField(Field field, Object fieldValue) throws IllegalArgumentException, IllegalAccessException {
        field.set(this, fieldValue);
    }
    
}
