/**
 * Source file created in 2012 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo.diff;

import java.io.Serializable;
import java.util.Collection;

import org.apache.commons.lang3.builder.HashCodeBuilder;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ValueObject;


/**
 * Represent a difference between two values of a field within a ValueObject.
 */
public class GroupDifference extends Difference implements Serializable {

    private static final long serialVersionUID = 1L;
   
    //TFM - do we need to name this something different, since this is not an override?
    private Collection oldValue;
    private Collection newValue;

    /**
     * Construct a new GroupDifference.
     * 
     * @param fieldKey FieldKey representing the field for this difference
     * @param oldValue Object old value before the change
     * @param newValue Object new, current value
     */
    public GroupDifference(FieldKey fieldKey, Object oldValue, Object newValue) {

        super(fieldKey, oldValue, newValue);
        this.oldValue = (Collection)oldValue;
        this.newValue = (Collection)newValue;           
    }    
    
    
    /**
     * Gets the old value.
     *
     * @return the oldValue
     */
    public Collection getOldValue() {
    
        return oldValue;
    }

    
    /**
     * Sets the old value.
     *
     * @param oldValue the oldValue to set
     */
    public void setOldValue(Collection oldValue) {
    
        this.oldValue = oldValue;
    }

    
    /**
     * Gets the new value.
     *
     * @return the newValue
     */
    public Collection getNewValue() {
    
        return newValue;
    }

    
    /**
     * Sets the new value.
     *
     * @param newValue the newValue to set
     */
    public void setNewValue(Collection newValue) {
    
        this.newValue = newValue;
    }

    /**
     * This method returns the short string representation of this object.
     * @param value value to get short string for
     * @return short string for the given value
     */
    protected String getShortString(Object value) {
        if (value == null) {

            return NO_VALUE;
        } else if (value instanceof Collection) {
            Object[] values = ((Collection) value).toArray();

            if (values.length <= 0) {
                
                return NO_SELECTION;
            } else if (values.length == 1) {
                
                return getShortString(values[0]);
            }

            StringBuffer sb = new StringBuffer();

            for (Object oNext : values) {
                
                sb.append(getShortString(oNext) + NEW_LINE);
            }

            return limitLengthOfString(sb.toString(), MAX_LIST_STRING_LEN_CHARS);
        }else if (value instanceof ValueObject) {
            return limitLengthOfString(((ValueObject) value).toShortString());
        }else {
            return limitLengthOfString(value.toString());
        }
    }    
    
    /**
     * This method returns the short string representation of this object.
     * @param value value to get short string for
     * @param isNewValue true if newValue request
     * @return short string for the given value
     */
    protected String getShortString(Object value, Boolean isNewValue) {
        if (value == null) {
            return NO_VALUE;
        }
        if (isNewValue){
            return (limitLengthOfString(getShortString(getNewValue())));
        }
        else{
            return (limitLengthOfString(getShortString(getOldValue()))); 
        }
    }


    /**
     * HashCode built by Jakarta Commons HasCodeBuilder.
     * 
     * @return int hash
     * 
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {

        return HashCodeBuilder.reflectionHashCode(this);
    }

    /**
     * toString returned by Jakarta Commons ToStringBuilder.
     * 
     * @return String properties and values of the ValueObject
     * 
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {

        return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
    }
}
