/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.vo.validator;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.time.DateUtils;

import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;


/**
 * validates the product copay tier
 */
public class ProductCopayTierValidator extends AbstractValidator<Collection<CopayTierAssocVo>> {

    /**
     * validate
     * 
     * @param target the copay tier
     * @param errors the errors collection
     * 
     */
    @Override
    public void validate(Collection<CopayTierAssocVo> target, Errors errors) { 
        if (isNull(target)) {
            rejectIfNull(target, errors, FieldKey.COPAY_TIER_ASSOC);
            return;
        }

        for (CopayTierAssocVo copayTierAssoc : target) {
            if (copayTierAssoc == null) {
                errors.addError(FieldKey.COPAY_TIER_ASSOC, ErrorKey.COMMON_EMPTY, new Object[] {FieldKey.COPAY_TIER_ASSOC});
                return;
            } else if (copayTierAssoc.getCopayTier() == null && copayTierAssoc.getEffectiveDate() == null) {
                errors.addError(FieldKey.COPAY_TIER_ASSOC, ErrorKey.COMMON_EMPTY, new Object[] {FieldKey.COPAY_TIER_ASSOC});
                return;
            } else if (copayTierAssoc.getCopayTier() == null ) {
                errors.addError(FieldKey.COPAY_TIER_ASSOC, ErrorKey.COMMON_EMPTY, new Object[] {FieldKey.COPAY_TIER_ASSOC});
                return;
            }
        }
        
        List<CopayTierAssocVo> list = new ArrayList<CopayTierAssocVo>(target);
        String previousCopayTierLevel = "";
        
        for (int index=0; index < list.size(); index++) {
            CopayTierAssocVo copayTier = list.get(index);
            Date effectiveDate = copayTier.getEffectiveDate();
            String copayTierLevel = copayTier.getCopayTier().getTierLevel();
            if (copayTierLevel.equals(previousCopayTierLevel)) {
                errors.addError(FieldKey.COPAY_TIER_ASSOC, 
                    ErrorKey.COPAY_TIER_DUPLICATED,new Object[] {FieldKey.COPAY_TIER_ASSOC}); 
                      return;
            }
            if (copayTierLevel!=null && effectiveDate==null) {
                errors.addError(FieldKey.COPAY_TIER_ASSOC, 
                    ErrorKey.COPAY_TIER_WITHOUT_EFFECTIVE_DATE,new Object[] {FieldKey.COPAY_TIER_ASSOC}); 
                      return;
            }
            
            if (copayTierLevel!=null && effectiveDate!=null && copayTier.getEndDate()!=null) {
                if (copayTier.getEndDate().compareTo(effectiveDate) < 0) {
                errors.addError(FieldKey.COPAY_TIER_ASSOC, 
                    ErrorKey.COPAY_TIER_ENDDATE_CANNOT_BE_PRIOR_THAN_THE_EFFECTIVEDATE,
                        new Object[] {FieldKey.COPAY_TIER_ASSOC}); 
                      return;
                }
            }
            
            if (list.size() > 1 && index > 0) {
                Date previousEndDate = list.get(index-1).getEndDate();
                if (previousEndDate == null) {
                    list.get(index-1).setEndDate(DateUtils.addDays(effectiveDate, -1));
                } else {
                    Date dayBeforeEffectiveDate = DateUtils.addDays(effectiveDate, -1);
                    if (previousEndDate.compareTo(dayBeforeEffectiveDate) != 0 ) {
                      errors.addError(FieldKey.COPAY_TIER_ASSOC, 
                      ErrorKey.COPAY_TIER_INVALID_EFFECTIVEDATE_AND_ENDDATE_ASSOC,new Object[] {FieldKey.COPAY_TIER_ASSOC}); 
                        return;
                    }
                }
            }
            previousCopayTierLevel = copayTierLevel;
        }
    }
}
