/**
 * Source file created in 2012 by Southwest Research Institute
 */

package gov.va.med.pharmacy.peps.common.utility.test;

import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;

/**
 * Unit tests for {@link ConfigFileUtility}.
 */
public class ConfigFileUtilityTest {

    private ConfigFileUtility cfu = new ConfigFileUtility();

    /**
     * testTimeout. Value is actually retrieved from the PPS-NConfig.xml file.
     *
     */
    @Test
    public void testTimeout() {

        assertEquals("Config timeout specification", 1000, cfu.getTimeout());
    }

    /**
     * testDivision.
     *
     */
    @Test
    public void testDivision() {

        assertEquals("Config division specification", ConfigFileUtility.DEFAULT_DIVISION, cfu.getDivision());
    }

    /**
     * testProxy.
     *
     */
    @Test
    public void testProxy() {

        assertEquals("Config NDFProxyUserIEN specification", ConfigFileUtility.DEFAULT_PROXY, cfu.getNdfProxyUserDUZ());
    }

    /**
     * testConnSpec.
     *
     */
    @Test
    public void testConnSpec() {

        assertEquals("Config connection spec name specification", ConfigFileUtility.DEFAULT_CONNSPEC,
            cfu.getConnectionSpecName());
    }

    /**
     * testRpcTimeout.
     *
     */
    @Test
    public void testRpcTimeout() {

        assertEquals("Config RPC Timeout specification", ConfigFileUtility.DEFAULT_RPCTIMEOUT, cfu.getRpcTimeout());
    }

    /**
     * testFdbLocation, value is being retrieved from the PPS-NConfig.xml file.
     *
     */
    @Test
    public void testFdbLocation() {

        assertThat(cfu.getFdbImageLocation(), notNullValue());
    }

    /**
     * Tests that {@link ConfigFileUtility#getPersistentDirectory()} is not null. We can't check that it exists because it will
     * only exist when running in WebLogic.
     */
    @Test
    public void testPersistentDirectory() {
        assertThat(cfu.getPersistentDirectory(), notNullValue());
    }

    /**
     * testIsaacCodingConcept. Verify that the value is retrieved from the PPS-NConfig.xml file.
     */
    @Test
    public void testIsaacCodingConcept() {
        assertThat(cfu.getIsaacCodingConceptURL(), notNullValue());
    }

    /**
     * testIsaacCodingComponent. Verify that the value is retrieved from the PPS-NConfig.xml file.
     */
    @Test
    public void testIsaacCodingComponent() {
        assertThat(cfu.getIsaacCodingComponentURL(), notNullValue());
    }
    
    /**
     * testIsaacCodingDescriptions. Verify that the value is retrieved from the PPS-NConfig.xml file.
     */
    @Test
    public void testIsaacCodingDescriptions() {
        assertThat(cfu.getIsaacCodingDescriptionsURL(), notNullValue());
    }    
}
