/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.common.utility.test.generator;


import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import gov.va.med.pharmacy.peps.common.utility.DateTimeUtility;
import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;
import gov.va.med.pharmacy.peps.common.vo.CopayTierVo;


/**
 * Generate a list of Product Copay Tier Associations for unit tests.
 */
public class CopayTierAssocVoGenerator extends VoGenerator<CopayTierAssocVo> {

    /**
     * Generate a list of CopayTierAssocVo.
     * 
     * @return List of CopayTierAssocVo
     */
    protected final List<CopayTierAssocVo> generate() {
        List<CopayTierAssocVo> testData = new ArrayList<CopayTierAssocVo>();
        CopayTierGenerator copayTierGenerator = new CopayTierGenerator();
        List<CopayTierVo> copayTierList = copayTierGenerator.generate();
        
        LocalDate lastEffectiveDate = LocalDate.now(); 
        int intNumOfCopayTiersToAdd = 4;
        int EFFECTIVE_DATE_DAYS = 30;
        int copayTierIndex = 0;
        for(int i=0; i<intNumOfCopayTiersToAdd; i++){
            CopayTierAssocVo copayTierAssocVo = new CopayTierAssocVo();
            copayTierAssocVo.setId("90" + copayTierIndex);
            copayTierAssocVo.setCopayTier(copayTierList.get(copayTierIndex++));
            
            LocalDate effectiveDate = lastEffectiveDate.plusDays(EFFECTIVE_DATE_DAYS);
            copayTierAssocVo.setEffectiveDate(Date.from(DateTimeUtility.toInstant(effectiveDate)));
            
            if (i < intNumOfCopayTiersToAdd - 1){
                //not the last one, so add an end date
                copayTierAssocVo.setEndDate(
                    Date.from(DateTimeUtility.toInstant(effectiveDate.plusDays(EFFECTIVE_DATE_DAYS-1))));
            }

            lastEffectiveDate = effectiveDate;
            testData.add(copayTierAssocVo);
        }               
        
        return testData;
    }
    
    /**
     * doInitialization.
     */
    @Override
    protected void doInitialization() {
        
    }
}
