package gov.va.med.pharmacy.peps.common.vo;

import static org.junit.Assert.*;

import java.util.Arrays;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;

/**
 * Unit tests for {@link ModificationSummaryVo}.
 */
public class ModificationSummaryVoTest {
  
  /**
   * Just for convenience since multiple tests use this.
   */
  private final ProductGenerator productGenerator = new ProductGenerator();

  @Test
  public void testShouldTriggerRematch_False() {
    final ModificationSummaryVo modSummary = new ModificationSummaryVo(
        productGenerator.getFirst(),
        Arrays.asList(createModDifference(new Difference(FieldKey.FDA_MED_GUIDE, "old", "new"))),
        null);

    assertFalse(modSummary.shouldTriggerRematch());
  }

  @Test
  public void testShouldTriggerRematch_True() {
    // Just test one case. Other tests test the complete set of cases.
    final ModificationSummaryVo modSummary = new ModificationSummaryVo(
        productGenerator.getFirst(),
        Arrays.asList(createModDifference(new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE))),
        null);

    assertTrue(modSummary.shouldTriggerRematch());
  }
  
  /**
   * Convenience method to create a {@link ModDifferenceVo} in one line.
   * @return not fully populated
   */
  private ModDifferenceVo createModDifference(final Difference difference) {
    final ModDifferenceVo modDiff = new ModDifferenceVo();
    modDiff.setDifference(difference);
    modDiff.setModRequestItemStatus(RequestItemStatus.PENDING);
    return modDiff;
  }

}
