package gov.va.med.pharmacy.peps.common.vo;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.HashSet;

import org.junit.Test;

/**
 * Unit tests for {@link RequestVo}.
 */
public class RequestVoTest {
  
  /**
   * The ID used for the next generated {@link ModDifferenceVo}. See {@link #makeTestModDifference(RequestItemStatus)}.
   */
  private static int nextModDiffId = 1;
  
  @Test
  public void testGetApprovedDetails() {
    
    /* Setup */
    final RequestVo objUnderTest = new RequestVo();
    final ModDifferenceVo approvedDiff1 = makeTestModDifference(RequestItemStatus.APPROVED);
    final ModDifferenceVo approvedDiff2 = makeTestModDifference(RequestItemStatus.APPROVED);
    objUnderTest.setRequestDetails(Arrays.asList(
        approvedDiff1,
        makeTestModDifference(RequestItemStatus.REJECTED),
        makeTestModDifference(RequestItemStatus.PENDING),
        approvedDiff2));
    
    /* Behavior & Verification */
    // Use a Set because we don't care about iteration order.
    final HashSet<ModDifferenceVo> expectedDiffs = new HashSet<>(Arrays.asList(approvedDiff1, approvedDiff2));
    assertThat(new HashSet<>(objUnderTest.getApprovedDetails()), equalTo(expectedDiffs));
  }
  
  /**
   * Constructs a mostly-empty {@link ModDifferenceVo} with the given status. Each object returned will have a different
   * ID. Thread safe.
   * @return a new object every time
   */
  private static synchronized ModDifferenceVo makeTestModDifference(final RequestItemStatus status) {
    final ModDifferenceVo diff = new ModDifferenceVo();
    diff.setId(Integer.toString(nextModDiffId++));
    diff.setModRequestItemStatus(status);
    return diff;
  }

}
