package gov.va.med.pharmacy.peps.domain.common.capability;

import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

/**
 * Domain Capability for Compliance Sites.
 */
public interface ComplianceSiteDomainCapability {
    /**
     * Create Compliance Site in the database.
     * 
     * @param complianceSiteVo ComplianceSiteVo to insert
     * @param user {@link UserVo} performing the action
     * @return ComplianceSiteVo the inserted ComplianceSiteVo
     */
    public ComplianceSiteVo create(ComplianceSiteVo complianceSiteVo, UserVo user);
    
    /**
     * Update the given ComplianceSiteVo.
     * 
     * @param complianceSiteVo ComplianceSiteVo to update
     * @param user {@link UserVo} performing the action
     * @return ComplianceSiteVo updated ComplianceSiteVo
     */
    public ComplianceSiteVo update(ComplianceSiteVo complianceSiteVo, UserVo user);

    /**
     * Delete the specified ComplianceSiteVo.
     * 
     * @param complianceSiteVo ComplianceSiteVo to update
     * @param user {@link UserVo} performing the action
     */        
    void delete(ComplianceSiteVo complianceSiteVo, UserVo user);    

    /**
     * Retrieve the list of Compliance Sites by include/exclude indicator.
     *
     * @param includeExcludeIndicator the include exclude indicator
     * @return List<ComplianceSiteVo> List containing all the ComplianceSiteVo records matching the specified include/exclude
     *         indicator
     */
    public List<ComplianceSiteVo> retrieveByIncludeExcludeIndicator(String includeExcludeIndicator);

    /**
     * Save list of ComplianceSiteVos that all have the same includeExcludeIndicator.  The sites in the specified list should 
     * match the include exclude indicator specified.
     *
     * @param sitesToSave the sites to save
     * @param includeExcludeIndicator the include exclude indicator
     * @param user the user
     * @return the list
     */    
    List<ComplianceSiteVo> saveList(List<ComplianceSiteVo> sitesToSave, String includeExcludeIndicator, UserVo user);

}
