/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.domain.common.capability;


import java.util.List;

import gov.va.med.pharmacy.peps.common.exception.AuthenticationException;
import gov.va.med.pharmacy.peps.common.vo.UserVo;


/**
 * Perform CRUD operations on user
 */
public interface UserDomainCapability {

    /**
     * Retrieve all possible users
     * 
     * @return List of Generic Names
     */
    List<UserVo> retrieveAll();

    /**
     * Retrieves a user given a vauid and username.
     * 
     * @param vauid the vauid of the user to retrieve
     * @return matching user or null if none is found
     */
    UserVo retrieve(Long vauid);

    /**
     * Insert the given UserVo.
     * 
     * @param user UserVo
     * @return UserVo with new ID
     */
    UserVo create(UserVo user);
    
    /**
     * Return true if a {@link UserVo} with the given ID exists.
     * 
     * @param id Long
     * @return boolean
     */
    boolean exists(Long id);

    /**
     * Update the given UserVo.
     * 
     * @param userVo UserVo
     * @return UserVo
     */
    UserVo update(UserVo userVo);

    /**
     * Update the given user's preferences.
     * 
     * @param userVo UserVo
     * @return UserVo
     */
    UserVo updatePreferences(UserVo userVo);
}
