package gov.va.med.pharmacy.peps.domain.common.capability.impl;


import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;

import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.validator.ErrorKey;
import gov.va.med.pharmacy.peps.common.vo.validator.Errors;
import gov.va.med.pharmacy.peps.common.vo.validator.ValidationError;
import gov.va.med.pharmacy.peps.common.vo.validator.ValidationErrorLevel;
import gov.va.med.pharmacy.peps.domain.common.capability.FormularyDesignatorDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.DataAccessObject;
import gov.va.med.pharmacy.peps.domain.common.dao.EplFormularyDesignatorDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplFormularyDesignatorDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.FormularyDesignatorConverter;
import gov.va.med.pharmacy.peps.domain.common.utility.querybuilder.HqlBuilder;


/**
 * Perform CRUD operations on Formulary Designator.
 */
public class FormularyDesignatorDomainCapabilityImpl extends
        ManagedDataDomainCapabilityImpl<FormularyDesignatorVo, EplFormularyDesignatorDo>
        implements FormularyDesignatorDomainCapability {
    
    /**
     * The data access object for Formulary Designator.
     */
    private EplFormularyDesignatorDao eplFormularyDesignatorDao;
    
    /**
     * The converter for Formulary Designator.
     */
    private FormularyDesignatorConverter formularyDesignatorConverter;
    
    public final EplFormularyDesignatorDao getEplFormularyDesignatorDao() {
        return eplFormularyDesignatorDao;
    }

    
    public final void setEplFormularyDesignatorDao(EplFormularyDesignatorDao eplFormularyDesignatorDao) {
        this.eplFormularyDesignatorDao = eplFormularyDesignatorDao;
    }

    
    public final FormularyDesignatorConverter getFormularyDesignatorConverter() {
        return formularyDesignatorConverter;
    }

    
    public final void setFormularyDesignatorConverter(FormularyDesignatorConverter formularyDesignatorConverter) {
        this.formularyDesignatorConverter = formularyDesignatorConverter;
    }  
    
    /**
     * Checks whether formularyDesignatorVo with associated products is about to be inactivated.
     * Constructs HQL to query the Products table and determine
     * if there are products associated with the formulary designator.
     *  
     * @param formularyDesignatorVo to check for active product dependencies
     * @param user {@link UserVo} performing the action
     * @throws ValidationException if active products exist which are associated with the given
     * formularyDesignatorVo
     */
    @Override
    public Errors checkForInactivateWarnings(FormularyDesignatorVo formularyDesignatorVo) {
        Errors warnings = new Errors();
        Session session = this.getDataAccessObject().getCurrentSession();
        StringBuffer sql = new StringBuffer();
        sql.append(
            HqlBuilder.create("SELECT item FROM ").append(EplProductDo.class)
                .append(PPSConstants.ITEM)
                .append(" WHERE FORMULARY_DESIGNATOR_FK = " + formularyDesignatorVo.getId())
                .append(" AND ITEM_STATUS = 'ACTIVE'")
        );
        
        Query query = session.createQuery(sql.toString());
        query.setMaxResults(1);
        
        @SuppressWarnings("rawtypes")
        List products = query.list();
        
        if (products != null && products.size() > 0) {
            warnings.addError(
                new ValidationError(
                    ErrorKey.INACTIVE_FORMULARY_DESIGNATOR_HAS_PRODUCTS_ASSIGNED, 
                    ValidationErrorLevel.WARNING));
        }  
        
        return warnings;
    }    
    
    /**
     * Create Hibernate {@link Criteria} that will find an item by its
     * uniqueness fields.
     * 
     * @param item
     *            {@link ManagedItemVo} for which to create uniqueness
     *            {@link Criteria}
     * @return {@link Criteria}
     */
    @Override
    protected final Criteria createUniquenessCriteria(FormularyDesignatorVo item) {
        Criteria criteria = getDataAccessObject().getCriteria();
        criteria.add(Restrictions.ilike(EplFormularyDesignatorDo.NAME,
                item.getName()));
        return criteria;
    }

    /**
     * Get the {@link DataAccessObject} that this FormularyDesignatorDomainCapabilityImpl uses.
     * 
     * @return {@link DataAccessObject}
     */
    public final DataAccessObject<EplFormularyDesignatorDo, Long> getDataAccessObject() {
        return eplFormularyDesignatorDao;
    }

    /**
     * Return the {@link Converter} instance for this capability.
     * 
     * @return {@link Converter}
     */
    public final FormularyDesignatorConverter getConverter() {
        return formularyDesignatorConverter;
    }

}
