package gov.va.med.pharmacy.peps.domain.common.model;

import java.util.Date;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

/**
 * Represents a compliance report site list include or exclude record.
 */
public class EplComplianceSiteDo extends DataObject {

    /** Name of Station Number property. */
    public static final String STATION_NUMBER = "stationNumber";
    /** Name of Include Exclude Indicator property. */
    public static final String INCLUDE_EXCLUDE_INDICATOR = "includeExcludeIndicator";

    /**
     * Serial Version UID.
     */
    private static final long serialVersionUID = 1L;

    /** The station number. */
    private String stationNumber;

    /** Indicator whether the record is an include or an exclude. */
    private String includeExcludeIndicator;

    /**
     * Instantiates a new compliance site do.
     *
     * @param stationNumber the station number
     * @param includeExcludeIndicator the include exclude indicator
     */
    public EplComplianceSiteDo(String stationNumber, String includeExcludeIndicator) {
        super();
        this.stationNumber = stationNumber;
        this.includeExcludeIndicator = includeExcludeIndicator;
    }

    /**
     * Default Constructor.
     */
    public EplComplianceSiteDo() {
        super();
    }

    /**
     * Constructor that takes super fields.
     * @param createdBy Username who originally created the record
     * @param createdDtm Date record was originally created
     * @param lastModifiedBy Username who last modified the record
     * @param lastModifiedDtm Date record was last created
     * @param minimallyPopulated Denotes that the record is minimally populated
     */
    public EplComplianceSiteDo(String createdBy, Date createdDtm, String lastModifiedBy, Date lastModifiedDtm,
        boolean minimallyPopulated) {
        super(createdBy, createdDtm, lastModifiedBy, lastModifiedDtm, minimallyPopulated);
    }

    @Override
    public boolean equals(Object other) {
        if ((this == other)) {
            return true;
        }

        if ((other == null)) {
            return false;
        }

        if (!(other instanceof EplComplianceSiteDo)) {
            return false;
        }

        EplComplianceSiteDo castOther = (EplComplianceSiteDo) other;

        return ((this.getStationNumber() == castOther.getStationNumber()) || (this.getStationNumber() != null
            && castOther.getStationNumber() != null && this.getStationNumber().equals(castOther.getStationNumber())))
            && ((this.getStationNumber() == castOther.getStationNumber()) || (this.getStationNumber() != null
                && castOther.getStationNumber() != null && this.getStationNumber().equals(castOther.getStationNumber())));
    }

    @Override
    public int hashCode() {
        int result = 17;

        result = 37 * result + (getStationNumber() == null ? 0 : this.getStationNumber().hashCode());

        return result;
    }

    /**
     * Returns a String representation of this instance of the object.
     * 
     * @return String properties and values of the ValueObject
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ToStringBuilder.reflectionToString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    /**
     * Gets the station number.
     *
     * @return the stationNumber
     */
    public final String getStationNumber() {
        return stationNumber;
    }

    /**
     * Sets the station number.
     *
     * @param stationNumber the stationNumber to set
     */
    public final void setStationNumber(String stationNumber) {
        this.stationNumber = stationNumber;
    }

    /**
     * Gets the include exclude indicator.
     *
     * @return the includeExcludeIndicator
     */
    public String getIncludeExcludeIndicator() {
        return includeExcludeIndicator;
    }

    /**
     * Sets the include exclude indicator.
     *
     * @param includeExcludeIndicator the includeExcludeIndicator to set
     */
    public void setIncludeExcludeIndicator(String includeExcludeIndicator) {
        this.includeExcludeIndicator = includeExcludeIndicator;
    }

}
