package gov.va.med.pharmacy.peps.domain.common.model.dto;

import java.math.BigDecimal;
import java.time.Instant;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.apache.commons.lang3.builder.ToStringStyle;

import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;

/**
 * <p>
 * Data Transfer Object representing a product with a future inactivation. Perhaps {@link EplProductDo} could be used in
 * place of this class if we could convert the queries into HQL.
 * </p>
 * <p>
 * Since this class is not used for updates, it would be better to make all fields final. But having setters allows us
 * to user Hibernate's aliasToBean transformers.
 * </p>
 */
public final class FutureInactiveProductDto {
  private BigDecimal eplId;
  private String vaProductName;
  private String cmopId;
  private Instant futureDate;

  public BigDecimal getEplId() {
    return eplId;
  }

  public void setEplId(final BigDecimal pEplId) {
    eplId = pEplId;
  }

  public String getVaProductName() {
    return vaProductName;
  }

  public void setVaProductName(final String pVaProductName) {
    vaProductName = pVaProductName;
  }

  public String getCmopId() {
    return cmopId;
  }

  public void setCmopId(final String pCmopId) {
    cmopId = pCmopId;
  }

  public Instant getFutureDate() {
    return futureDate;
  }

  public void setFutureDate(Instant pFutureDate) {
    futureDate = pFutureDate;
  }

  /**
   * Converts the given String to the future date. This setter only exists for Hibernate.
   * @param futureDateString a String containing milliseconds since the epoch.
   */
  public void setFutureDateString(final String futureDateString) {
    futureDate = Instant.ofEpochMilli(Long.parseLong(futureDateString));
  }
  
  /**
   * Returns a string containing all of the object's public properties.
   */
  @Override
  public String toString() {
    return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
        .append(eplId)
        .append(vaProductName)
        .append(cmopId)
        .append(futureDate)
        .toString();
  }
}
