

package gov.va.med.pharmacy.peps.domain.common.utility.converter;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import gov.va.med.pharmacy.peps.common.object.VistaStatus;
import gov.va.med.pharmacy.peps.common.vo.EplVistaUpdateVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfUpdateFileDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplVistaUpdateStatusDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfUpdateFileDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVistaStatusDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVistaUpdateDo;

/**
 * Converter between {@link EplVistaUpdateVo}s and {@link EplVistaStatusDo}s.
 */
@Component
public class EplVistaUpdateConverter extends Converter<EplVistaUpdateVo, EplVistaUpdateDo> {
  
    private final EplVistaUpdateStatusDao vistaUpdateStatusDao;
    private final EplNdfUpdateFileDao eplNdfUpdateFileDao;
  
  /**
   * Constructs an instance with the required properties.
   * @param pVistaUpdateStatusDao the {@link EplVistaUpdateStatusDao} to use.
   * @param pEplNdfUpdateFileDao the {@link EplNdfUpdateFileDao} to use.
   */
  @Autowired
  public EplVistaUpdateConverter(
      final EplVistaUpdateStatusDao pVistaUpdateStatusDao, final EplNdfUpdateFileDao pEplNdfUpdateFileDao) {
    vistaUpdateStatusDao = pVistaUpdateStatusDao;
    eplNdfUpdateFileDao = pEplNdfUpdateFileDao;
  }

    @Override
    protected EplVistaUpdateDo toDataObject(final EplVistaUpdateVo valueObj) {
        final EplVistaUpdateDo updateDo = new EplVistaUpdateDo();

        updateDo.setComments(valueObj.getComments());
        updateDo.setEplVistaStatus(retrieveStatus(valueObj.getStatus()));
        updateDo.setSite(valueObj.getSite());
        updateDo.setEplNdfUpdateFileDo(retrieveFile(valueObj.getFileName()));
        updateDo.setTransmissionDate(valueObj.getTransmissionDate());
        return updateDo;
    }
    
    /**
     * Retrieves the {@link EplVistaStatusDo} from the database by name.
     * @throws IllegalArgumentException if the given name does not represent a valid status
     */
    private EplVistaStatusDo retrieveStatus(final String name) {
        final VistaStatus vistaStatus = VistaStatus.valueOf(name); // valueOf() throws IllegalArgumentException
        return vistaUpdateStatusDao.getStatusByName(vistaStatus);
    }
    
    /**
     * Retrieves the {@link EplNdfUpdateFileDo} from the database by name.
     * @throws IllegalArgumentException if no file with the given name could be found
     * @return never null
     */
    private EplNdfUpdateFileDo retrieveFile(final String name) {
        final EplNdfUpdateFileDo fileDo = eplNdfUpdateFileDao.retrieveByName(name);
        if (fileDo == null) {
            throw new IllegalArgumentException("No update file with that name exists.");
        }
        return fileDo;
    }

    @Override
    protected EplVistaUpdateVo toValueObject(EplVistaUpdateDo data) {
        EplVistaUpdateVo eplVistaUpdateVo = new EplVistaUpdateVo();
        eplVistaUpdateVo.setComments(data.getComments());
        eplVistaUpdateVo.setFileName(data.getEplNdfUpdateFileDo().getFileName());
        eplVistaUpdateVo.setSite(data.getSite());
        eplVistaUpdateVo.setStatus(data.getEplVistaStatus().getStatusName());
        eplVistaUpdateVo.setTransmissionDate(data.getTransmissionDate());
        
        return eplVistaUpdateVo;
    }

}
