package gov.va.med.pharmacy.peps.domain.common.utility.converter;


import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ValueObject;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.model.DataObject;
import gov.va.med.pharmacy.peps.domain.common.model.FileBasedSiteDo;


/**
 * Convert to/from {@link VistaSiteVo} and {@link FileBasedSiteDo}.
 */
public class VistaSiteConverter extends Converter<VistaSiteVo, FileBasedSiteDo> {

    /**
     * Fully copies data from the given VistaSiteVo into a {@link DataObject}.
     * This is an unsupported operation.  FileBasedSiteDo is read-only, so there is no reason to convert from VistaSiteVo.
     * 
     * @param data {@link ValueObject} to convert
     * @return fully populated {@link DataObject}
     * 
     */
    @Override
    protected FileBasedSiteDo toDataObject(VistaSiteVo data) {
        throw new UnsupportedOperationException(
            "FileBasedSiteDo is read-only, so there is no reason to convert from VistaSiteVo.");        
    }

    /**
     * Fully copies data from the given {@link DataObject} into a VistaSiteVo.
     * <p>
     * Parent objects, if any, are minimally populated. Child objects, if any, are not populated at all. Any remaining
     * aggregated {@link ManagedItemVo} are minimally populated. If any of these item types are intended to be fully
     * populated, the appropriate {@link Converter#convert(DataObject)} should be called.
     * 
     * @param data {@link DataObject} to convert
     * @return fully populated VistaSiteVo
     * 
     */
    @Override
    protected VistaSiteVo toValueObject(FileBasedSiteDo data) {
        if (data == null){
            return null;
        }
        return new VistaSiteVo(data.getStationNumber(), data.getName());

    }   
       
}
