package gov.va.med.pharmacy.peps.service.common.update.impl;


import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.PackageTypeVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.utility.update.item.PackageTypeConverter;
import gov.va.med.pharmacy.peps.external.common.vo.outbound.common.ItemAction;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessor;


/**
 * A NdfFileSyncProcessor implementation that handles PackageTypeVo objects.
 * 
 * @author vhaisaduffg
 *
 */
public class PackageTypeNdfFileSyncProcessor extends AbstractSimpleNdfFileSyncProcessor implements NdfFileSyncProcessor {

    /** FieldKeys of the fields that, if modified, would be processed by the modification logic of this processor. */
    public static final Set<FieldKey> FIELDS = Collections.unmodifiableSet(new LinkedHashSet<FieldKey>(Arrays
        .asList(
            FieldKey.VALUE, 
            FieldKey.INACTIVATION_DATE, 
            FieldKey.ITEM_STATUS
            )));
    
    private static final String FILE_NUMBER = "50.608";
    private static final String PACKAGE_TYPE_IEN_FIELD_NUMBER = ".01";
    private static final String INACTIVATION_DATE_FIELD_NUMBER = "1";
    
    /**
     * Instantiates a new PackageTypeNdfFileSyncProcessor.
     *
     * @param pNdfFileSyncCapability the ndf file sync capability
     * @see AbstractSimpleNdfFileSyncProcessor#AbstractSimpleNdfFileSyncProcessor(Set, String, String)
     */
    public PackageTypeNdfFileSyncProcessor(NdfFileSyncCapability pNdfFileSyncCapability) {
        super(FIELDS, FILE_NUMBER, PACKAGE_TYPE_IEN_FIELD_NUMBER, pNdfFileSyncCapability);        
    }
    

    @Override
    public void processNew(ManagedItemVo managedItem, UserVo user) {
        PackageTypeVo packageTypeVo = (PackageTypeVo) managedItem;
        
        if (RequestItemStatus.APPROVED.equals(packageTypeVo.getRequestItemStatus()) && packageTypeVo.getValue() != null) {
            EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();

            // IEN
            diffObj.setVistaIen(packageTypeVo.getPackagetypeIen());
            diffObj.setNewValue(packageTypeVo.getValue()); 
            diffObj.setCreatedBy(user.getUsername());
            processInsert(diffObj); 
        }
    }

    @Override
    public void processModified(ManagedItemVo managedItem, UserVo user, Collection<Difference> differences) {
       
     Map<FieldKey, Difference> setDifference =  Difference.toDifferenceMap(differences);
     boolean hasDifference = PackageTypeConverter.hasNewOrModifiedFields(super.getFields(), setDifference, ItemAction.MODIFY);

     PackageTypeVo packageTypeVo = (PackageTypeVo) managedItem;

     if (hasDifference) {
         if (RequestItemStatus.APPROVED.equals(packageTypeVo.getRequestItemStatus()) && packageTypeVo.getValue() != null) {
             EplNdfOutgoingDifferences diffObj = new EplNdfOutgoingDifferences();

             diffObj.setVistaIen(packageTypeVo.getPackagetypeIen());
             diffObj.setVistaFieldNumber(INACTIVATION_DATE_FIELD_NUMBER);
             if(packageTypeVo.getInactivationDate() != null){
                 diffObj.setNewValue(MumpsConverter.convertDate(packageTypeVo.getInactivationDate()));                 
             }  
             else{
                 //passing an empty string because this is a non null field
                 diffObj.setNewValue(VistaFile.BLANK);                  
             }
             diffObj.setCreatedBy(user.getUsername());
             processUpdate(diffObj);
         }

     }

        
    } 
}
