/**
 * Source file created in 2011 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.domain.common.capability.test;


import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.junit.Assert.assertThat;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.capability.ComplianceSiteDomainCapability;

/**
 * JUnit test for ComplianceSiteDomainCapability.
 * 
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = { "classpath*:xml/spring/test/DomainContext.xml" })
@Transactional("transactionManager")
public class ComplianceSiteDomainCapabilityIT {

    private static final int EXPECTED_SECOND_INCLUDE_COUNT = 3;
   
    /** The compliance site domain capability. */
    @Autowired
    private ComplianceSiteDomainCapability complianceSiteDomainCapability;

    /**
     * Test retrieve by include exclude indicator.
     */
    @Test
    public void testRetrieveByIncludeExcludeIndicator(){
        UserVo user = new UserVo();
        user.setUsername("TestUser");
        ComplianceSiteVo site1 = new ComplianceSiteVo(new VistaSiteVo("SITE1", "SITE1 VAMC"),ComplianceSiteVo.INCLUDE);
        ComplianceSiteVo site2 = new ComplianceSiteVo(new VistaSiteVo("SITE2", "SITE2 VAMC"),ComplianceSiteVo.INCLUDE);
        ComplianceSiteVo site3 = new ComplianceSiteVo(new VistaSiteVo("SITE3", "SITE3 VAMC"),ComplianceSiteVo.EXCLUDE);
        ComplianceSiteVo site4 = new ComplianceSiteVo(new VistaSiteVo("SITE4", "SITE4 VAMC"),ComplianceSiteVo.EXCLUDE);
        
        complianceSiteDomainCapability.create(site1, user);
        complianceSiteDomainCapability.create(site2, user);
        complianceSiteDomainCapability.create(site3, user);
        complianceSiteDomainCapability.create(site4, user);
        
        List<ComplianceSiteVo> included = 
            complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE);
        assertThat(included, notNullValue());
        assertThat(included.size(), equalTo(2));

        List<ComplianceSiteVo> excluded = 
            complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.EXCLUDE);
        assertThat(excluded, notNullValue());
        assertThat(excluded.size(), equalTo(2));

        //Update site 3 to be included instead of excluded
        site3.setIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE);
        site3 = complianceSiteDomainCapability.update(site3, user);
        
        included = 
            complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE);
        assertThat(included, notNullValue());
        assertThat(included.size(), equalTo(EXPECTED_SECOND_INCLUDE_COUNT));

        excluded = 
            complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.EXCLUDE);
        assertThat(excluded, notNullValue());
        assertThat(excluded.size(), equalTo(1));
        
    }
    
    @Test
    public void testSaveList(){
        UserVo user = new UserVo();
        user.setUsername("TestUser");
        
        ComplianceSiteVo site1 = new ComplianceSiteVo(new VistaSiteVo("SITE1", "SITE1 VAMC"),ComplianceSiteVo.INCLUDE);
        ComplianceSiteVo site2 = new ComplianceSiteVo(new VistaSiteVo("SITE2", "SITE2 VAMC"),ComplianceSiteVo.INCLUDE);
        ComplianceSiteVo site3 = new ComplianceSiteVo(new VistaSiteVo("SITE3", "SITE3 VAMC"),ComplianceSiteVo.EXCLUDE);
        ComplianceSiteVo site4 = new ComplianceSiteVo(new VistaSiteVo("SITE4", "SITE4 VAMC"),ComplianceSiteVo.EXCLUDE);
        
        complianceSiteDomainCapability.create(site1, user);
        complianceSiteDomainCapability.create(site2, user);
        complianceSiteDomainCapability.create(site3, user);
        complianceSiteDomainCapability.create(site4, user);        
        
        ComplianceSiteVo siteToSave1 = new ComplianceSiteVo(new VistaSiteVo("SITE1", "SITE1 VAMC"),ComplianceSiteVo.INCLUDE);
        ComplianceSiteVo siteToSave2 = new ComplianceSiteVo(new VistaSiteVo("SITE5", "SITE5 VAMC"),ComplianceSiteVo.INCLUDE);
        List<String> expectedStationNumbersList = Arrays.asList(siteToSave1.getVistaSiteVo().getStationNumber(),
            siteToSave2.getVistaSiteVo().getStationNumber());
        
        
        
        List<ComplianceSiteVo> sitesToSave = Arrays.asList(siteToSave1, siteToSave2);
        
        List<ComplianceSiteVo> savedSites = complianceSiteDomainCapability.saveList(sitesToSave, 
            sitesToSave.get(0).getIncludeExcludeIndicator(), user);
        
        assertThat(savedSites, notNullValue());
        assertThat(savedSites.size(), equalTo(sitesToSave.size()));
        
        savedSites.stream().forEach(site -> assertThat(
            expectedStationNumbersList.contains(site.getVistaSiteVo().getStationNumber()), equalTo(true)));
        
        List<ComplianceSiteVo> includedSitesFromDB = 
            complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE);
        assertThat(includedSitesFromDB.size(), equalTo(savedSites.size()));
        
        includedSitesFromDB.stream().forEach(site -> assertThat(
            expectedStationNumbersList.contains(site.getVistaSiteVo().getStationNumber()), equalTo(true)));
        
    }
}
