package gov.va.med.pharmacy.peps.domain.common.model;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertThat;

import java.util.Date;

import org.apache.commons.lang3.time.DateUtils;
import org.junit.Test;


/**
 * JUnit test class for EplFdbAddDo.
 */
public class EplFdbAddDoTest {
  private static final Date FDB_CREATED_DATE = new Date();
  private static final Date CREATED_DATE = DateUtils.addDays(new Date(), 1);
  private static final Date MODIFIED_DATE = DateUtils.addDays(new Date(), 2);

  /**
   * Tests basic equals() and hashCode() correctness.
   */
  @Test
  public void testEqualsAndHashCode() {

    final EplFdbAddDo baselineDo = createDo();

    final EplFdbAddDo identicalDo = createDo();
    
    assertThat("should be equal to itself", baselineDo, equalTo(baselineDo));
    assertThat("identical object should be equal", baselineDo, equalTo(identicalDo));
    assertEqualsSymmetric(baselineDo, identicalDo);
    assertThat("identical object's hash code should be equal",
        identicalDo.hashCode(), equalTo(baselineDo.hashCode()));    

    //Now, cycle through each field, making it different from the baseline.
    EplFdbAddDo differentDo = createDo();
    differentDo.setNdc(baselineDo.getNdc() + "1");
    assertEqualsSymmetric(baselineDo, differentDo);
    assertUnequalOneFieldDiff("NDC", baselineDo, differentDo);
    
    differentDo = createDo();
    differentDo.setPackageSize(baselineDo.getPackageSize() + "1");
    assertUnequalOneFieldDiff("Package Size", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setPackageType(baselineDo.getPackageType() + "1");
    assertUnequalOneFieldDiff("Package Type", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setManufacturer(baselineDo.getManufacturer() + "1");
    assertUnequalOneFieldDiff("Manufacturer", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setLabelName(baselineDo.getLabelName() + "1");
    assertUnequalOneFieldDiff("Label Name", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setAddDesc(baselineDo.getAddDesc() + "1");
    assertUnequalOneFieldDiff("Add Desc.", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setTradeName(baselineDo.getTradeName() + "1");
    assertUnequalOneFieldDiff("Trade Name", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setFdbProductName(baselineDo.getFdbProductName() + "1");
    assertUnequalOneFieldDiff("FDB Product Name", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setGcnSeqno(baselineDo.getGcnSeqno() + 1);
    assertUnequalOneFieldDiff("GCN Seq No", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setFdbCreationDate(DateUtils.addDays(baselineDo.getFdbCreationDate(), 1));
    assertUnequalOneFieldDiff("FDB Creation Date", baselineDo, differentDo);
    
    differentDo = createDo();
    differentDo.setCreatedBy(baselineDo.getCreatedBy() + "1");
    assertUnequalOneFieldDiff("Created By", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setCreatedDtm(DateUtils.addDays(baselineDo.getCreatedDtm(), 1));
    assertUnequalOneFieldDiff("Created Date", baselineDo, differentDo);

    differentDo = createDo();
    differentDo.setLastModifiedBy(baselineDo.getLastModifiedBy() + "1");
    assertUnequalOneFieldDiff("Last Modified By", baselineDo, differentDo);
    
    differentDo = createDo();
    differentDo.setLastModifiedDtm(DateUtils.addDays(baselineDo.getLastModifiedDtm(), 1));
    assertUnequalOneFieldDiff("Last Modified Date", baselineDo, differentDo);
    
    
//    LOG.debug("baselineDo: {}", baselineDo);
//    LOG.debug("identicalTierChange: {}", identicalTierChange);
//    // Just make one field different.
//    LOG.debug("differentTierChange: {}", differentDo);
//    
//    assertThat("should be equal to itself", baselineDo, equalTo(baselineDo));
//    assertThat("identical object should be equal", identicalDo, equalTo(baselineDo));
//    assertThat("identical object's hash code should be equal",
//        identicalDo.hashCode(), equalTo(baselineDo.hashCode()));
//    assertThat("different object should not be equal", differentDo, not(equalTo(baselineDo)));
//    assertThat("different object's hash code should not be equal",
//        differentDo.hashCode(), not(equalTo(baselineDo.hashCode())));
  }

  /**
   * @param FDB_CREATED_DATE
   * @param CREATED_DATE
   * @param MODIFIED_DATE
   * @return
   */
  private EplFdbAddDo createDo() {
    final EplFdbAddDo baselineDo = new EplFdbAddDo("NDC", "PACK SIZE", "PACK TYPE", "Manufactured By", "LABEL", "ADD Desc", 
        "Trade Name", "FDB Prod Name", 123456L, FDB_CREATED_DATE, "Created By User", CREATED_DATE, "Modified By User",
        MODIFIED_DATE);
    return baselineDo;
  }

  /**
   * @param baselineDo
   * @param differentDo
   */
  private void assertUnequalOneFieldDiff(final String fieldDifferent, final EplFdbAddDo baselineDo, EplFdbAddDo differentDo) {
    assertThat("Should be not equal when " + fieldDifferent + " is changed", baselineDo, not(equalTo(differentDo)));
    assertThat("different object's hash code should not be equal when " + fieldDifferent + " is changed",
        differentDo.hashCode(), not(equalTo(baselineDo.hashCode())));
  }

  /**
   * Assert equals is symmetric.  Per the equals contract o1.equals(o2) should return the same as o2.equals(o1).
   *
   * @param do1 the do1
   * @param do2 the do2
   */
  private void assertEqualsSymmetric(EplFdbAddDo do1, EplFdbAddDo do2) {
    assertThat(do1.equals(do2), equalTo(do2.equals(do1)));
  }  
  
}
