package gov.va.med.pharmacy.peps.domain.common.utility;

import static org.easymock.EasyMock.anyObject;
import static org.easymock.EasyMock.eq;
import static org.easymock.EasyMock.expect;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.URISyntaxException;

import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIenGeneratorDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplPackageSizeDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplIenGeneratorDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplPackageSizeDo;

/**
 * Automated unit tests for {@link DDIPackageSizeReader}.
 */
public class PackageSizeFileReaderTest extends EasyMockSupport {
    private EplIenGeneratorDao mockIenGeneratorDao;
    private EplPackageSizeDao mockPackageSizeDao;
    
    /**
     * Constructs necessary dependencies as mocks for convenience.
     */
    @Before
    public void createMocks() {
        mockPackageSizeDao = strictMock(EplPackageSizeDao.class);
        mockIenGeneratorDao = strictMock(EplIenGeneratorDao.class);
    }
    
    @Test
    public void testProcessFile() throws URISyntaxException, IOException {

        /* Setup */

        // These constants must match the input file.
        final long highestIen = 8000;

        final UserVo testUser = new UserGenerator().getFirst();
        final PackageSizeFileReader psReader = new PackageSizeFileReader(
            mockPackageSizeDao, mockIenGeneratorDao, testUser);
        final File testFile = new File(getClass().getResource("/test_data/EPL_PACKAGE_SIZES.DAT").toURI());

        final EplIenGeneratorDo expectedIenGeneratorRecord =
            new EplIenGeneratorDo(EplPackageSizeDo.TABLE_NAME, highestIen, null, null);

        expectLookupAndInsert(testUser);
        expect(mockIenGeneratorDao.insert(expectedIenGeneratorRecord, testUser))
            .andReturn(null);  // Kludge: PackageSizeFileReader ignores the return value.

        replayAll();

        /* Behavior */
        
        assertThat(psReader.getHighestIenProcessed(), equalTo(PackageSizeFileReader.NO_IENS_PROCESSED));

        try (final RandomAccessFile raFile = new RandomAccessFile(testFile, "r")) {
            psReader.setFile(raFile);
            psReader.processFile();
        }
        
        /* Verification */
        assertThat(psReader.getHighestIenProcessed(), equalTo(highestIen));

        verifyAll();
    }
    private void expectLookupAndInsert(
        final UserVo testUser) {
        
        expect(mockPackageSizeDao.insert(anyObject(EplPackageSizeDo.class), eq(testUser)))
        .andReturn(null);  // Kludge: DDIFileReader ignores the return value.
    }   
}
