package gov.va.med.pharmacy.peps.domain.common.utility.converter;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.CoreMatchers.nullValue;
import static org.junit.Assert.assertThat;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.model.FileBasedSiteDo;

/**
 * JUnit test for VistaSiteConverter.
 */
public class VistaSiteConverterTest {


    /**
     * Test toDataObject - expects an UnsupportedOperationException.
     * @see VistaSiteConverter#toDataObject(VistaSiteVo)
     */
    @Test(expected=UnsupportedOperationException.class)
    public final void testToDataObjectVistaSiteVo() {
        VistaSiteVo vistaSiteVo = new VistaSiteVo("STATION", "NAME");
        new VistaSiteConverter().toDataObject(vistaSiteVo);
    }

    /**
     * Test toValueObject with a populated FileBasedSiteDo.
     * @see VistaSiteConverter#toValueObject(FileBasedSiteDo)
     */
    @Test
    public final void testToValueObjectFileBasedSiteDo() {
        FileBasedSiteDo fileBasedSiteDo = new FileBasedSiteDo();
        fileBasedSiteDo.setStationNumber("STATION");
        fileBasedSiteDo.setName("NAME");
        fileBasedSiteDo.setDeactivationDate("DEACTIVATIONDATE");
        fileBasedSiteDo.setFacilityTypeCode("FACILITYTYPECODE");
        
        VistaSiteVo vistaSiteVo = new VistaSiteConverter().toValueObject(fileBasedSiteDo);
        assertThat(vistaSiteVo, notNullValue());
        assertThat(vistaSiteVo.getStationNumber(), equalTo("STATION"));
        assertThat(vistaSiteVo.getName(), equalTo("NAME"));
    }

    /**
     * Test toValueObject with a null FileBasedSiteDo.
     * @see VistaSiteConverter#toValueObject(FileBasedSiteDo)
     */
    @Test
    public final void testToValueObjectFileBasedSiteDoNull() {
        FileBasedSiteDo fileBasedSiteDo = null;
       
        VistaSiteVo vistaSiteVo = new VistaSiteConverter().toValueObject(fileBasedSiteDo);
        assertThat(vistaSiteVo, nullValue());
    }
    
    
}
