package gov.va.med.pharmacy.peps.domain.common.utility.test;

import java.sql.Timestamp;
import java.util.Collection;
import java.util.Date;

import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import gov.va.med.pharmacy.peps.domain.common.model.DataObject;

/**
 * Base class for domain-level automated integration tests that pulls in the test Domain Context.
 */
@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(locations = {"classpath:xml/spring/test/DomainContext.xml"})
public abstract class AbstractDomainIT {

    /**
     * Replaces possible {@link java.sql.Timestamp}s in the given {@link DataObject} with {@link Date}s. This is necessary for
     * {@link DataObject#equals(Object)} to work properly. See the documentation for {@link Timestamp} for an explanation of
     * why.
     * 
     * @param dataObject the object to fix
     */
    public static void fixDates(final DataObject dataObject) {
        dataObject.setCreatedDtm(new Date(dataObject.getCreatedDtm().getTime()));
        dataObject.setLastModifiedDtm(new Date(dataObject.getLastModifiedDtm().getTime()));
    }

    /**
     * Performs {@link #fixDates(DataObject)} on all given objects.
     * 
     * @param dataObjects the objects to fix
     */
    public static void fixDates(final Iterable<? extends DataObject> dataObjects) {
        for (final DataObject dataObject : dataObjects) {
            fixDates(dataObject);
        }
    }
}
