package gov.va.med.pharmacy.peps.service.common.update;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.not;
import static org.junit.Assert.assertThat;

import java.time.LocalDate;
import java.util.Optional;

import org.apache.logging.log4j.Logger;
import org.junit.Test;

/**
 * Unit tests for {@link CopayTierChange}.
 */
public class CopayTierChangeTest {

  private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(
      CopayTierChangeTest.class);

  /**
   * Tests all public object properties.
   */
  @Test
  public void testAllProperties() {
    
    /* Setup */
    final String testProductIen = "10001";
    final String testProductName = "FLUCONAZOLE TEST";
    final String testCmopId = "F5000";
    final String testCopayTier = "127";
    final Optional<LocalDate> testEffectiveDate = Optional.of(LocalDate.of(2016, 11, 1));
    final Optional<LocalDate> testEndDate = Optional.of(LocalDate.of(2016, 11, 15));
    
    /* Behavior */
    final CopayTierChange testSubject = new CopayTierChange(
        testProductIen, testProductName, testCmopId, testCopayTier, testEffectiveDate, testEndDate);
    
    /* Verification */
    assertThat(testSubject.getProductIen(), equalTo(testProductIen));
    assertThat(testSubject.getProductName(), equalTo(testProductName));
    assertThat(testSubject.getCmopId(), equalTo(testCmopId));
    assertThat(testSubject.getCopayTier(), equalTo(testCopayTier));
    assertThat(testSubject.getEffectiveDate(), equalTo(testEffectiveDate));
    assertThat(testSubject.getEndDate(), equalTo(testEndDate));
    // We don't specify the contents of toString(). Just make sure that it's not empty.
    assertThat(testSubject.toString(), not(equalTo("")));
  }
  
  /**
   * Tests basic equals() and hashCode() correctness. Testing them for full correctness is quite
   * complicated.
   */
  @Test
  public void testEqualsAndHashCode() {
    final LocalDate effectiveDate = LocalDate.of(2016, 12, 2);
    final CopayTierChange baseTierChange = new CopayTierChange(
        "1", "Name1", "Cmop1", "1", Optional.of(effectiveDate), Optional.empty());
    LOG.debug("baseTierChange: {}", baseTierChange);
    final CopayTierChange identicalTierChange = new CopayTierChange(
        baseTierChange.getProductIen(),
        baseTierChange.getProductName(),
        baseTierChange.getCmopId(),
        baseTierChange.getCopayTier(),
        baseTierChange.getEffectiveDate(),
        baseTierChange.getEndDate());
    LOG.debug("identicalTierChange: {}", identicalTierChange);
    // Just make one field different.
    final CopayTierChange differentTierChange = new CopayTierChange(
        baseTierChange.getProductIen(),
        baseTierChange.getProductName(),
        baseTierChange.getCmopId(),
        "2",
        baseTierChange.getEffectiveDate(),
        baseTierChange.getEndDate());
    LOG.debug("differentTierChange: {}", differentTierChange);
    
    assertThat("should be equal to itself", baseTierChange, equalTo(baseTierChange));
    assertThat("identical object should be equal", identicalTierChange, equalTo(baseTierChange));
    assertThat("identical object's hash code should be equal",
        identicalTierChange.hashCode(), equalTo(baseTierChange.hashCode()));
    assertThat("different object should not be equal", differentTierChange, not(equalTo(baseTierChange)));
    assertThat("different object's hash code should not be equal",
        differentTierChange.hashCode(), not(equalTo(baseTierChange.hashCode())));
  }

}
