package gov.va.med.pharmacy.peps.service.common.capability;

import java.util.Objects;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import gov.va.med.pharmacy.peps.common.vo.RematchVo;

/**
 * Similar to {@link gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo}, but represents a rematch to save instead of a
 * difference.
 */
public class ModRematch {

    private Optional<RematchVo> optionalRematch;
    
    private String reason;
    
    /**
     * Constructs an instance, passing in all properties.
     * 
     * @param pOptionalRematch must not be null.
     * @param pReason may be null: will be converted to an empty string
     */
    public ModRematch(final Optional<RematchVo> pOptionalRematch, final String pReason) {
        optionalRematch = Objects.requireNonNull(pOptionalRematch);
        reason = StringUtils.defaultString(pReason);
    }

    /**
     * Returns the rematch to save. An empty optional here indicates that the existing rematch for the product should be
     * deleted.
     * @return never null
     */
    public Optional<RematchVo> getOptionalRematch() {
        return optionalRematch;
    }

    /**
     * Sets the rematch to save.
     * @param optionalRematch Must not be null.
     */
    public void setOptionalRematch(Optional<RematchVo> optionalRematch) {
        this.optionalRematch = Objects.requireNonNull(optionalRematch);
    }

    /**
     * Returns the rematch reason for recording in the audit history.
     * @return never null, but possibly empty
     */
    public String getReason() {
        return reason;
    }

    /**
     * Sets the rematch reason. See {@link #getReason()}.
     * @param reason May be null: will be converted to the empty string.
     */
    public void setReason(String reason) {
        this.reason = StringUtils.defaultString(reason);
    }

}
