/**
 * Source file created in 2008 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.service.common.capability;


import java.util.Collection;
import java.util.Optional;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.RequestCompletedException;
import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.RequestSummaryVo;
import gov.va.med.pharmacy.peps.common.vo.RequestVo;
import gov.va.med.pharmacy.peps.common.vo.SearchRequestCriteriaVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;


/**
 * Search and retrieve requests.
 * <p>
 * Local/National implement this differently. There is not a common implementation, only a common interface!
 */
public interface RequestCapability {

    /**
     * Retrieve request detail by ID.
     * 
     * @param id RequestVo ID to retrieve
     * @return RequestVo
     * @throws ItemNotFoundException if cannot find Request with given ID
     * @throws RequestCompletedException if the request has already been completed
     */
    RequestVo retrieve(String id) throws ItemNotFoundException, RequestCompletedException;

    /**
     * Retrieve all the requests for the specified item.
     * 
     * @param itemId the specified item's ID
     * @param itemType the specified item's entity type
     * @return the retrieved requests, never null.
     * @throws ItemNotFoundException if RequestVo with given ID cannot be found
     */
    Collection<RequestVo> retrieve(String itemId, EntityType itemType) throws ItemNotFoundException;

    /**
     * RequestCapability. Retrieve request by eplId 
     *  
     * @param itemId an enterprise level id
     * @param entityType entityType
     * @return RequestVo
     * 
     * @throws ItemNotFoundException if RequestVo with given ID cannot be found
     * 
     * @see gov.va.med.pharmacy.peps.domain.common.capability.RequestDomainCapability
     *      #retrieveRequestDetailById(java.lang.String)
     */
    RequestVo retrieveNonCompletedRequest(String itemId, EntityType entityType) throws ItemNotFoundException; 
    
    /**
     * Search for the pending requests
     * 
     * @param criteria for searchCriteria
     * @param user the user performing the search
     * @return Collection<RequestSummaryVo>
     * 
     * @throws ValueObjectValidationException if error in criteria data
     */
    Collection<RequestSummaryVo> search(SearchRequestCriteriaVo criteria, UserVo user) throws ValueObjectValidationException;

    /**
     * Returns the rematch, if any, associated with the given request.
     * @param request the request
     * @param item the item that the request is for
     * @return an Optional containing the associated rematch, if any. Never null.
     */
    Optional<RematchVo> getAssociatedRematch(final RequestVo request, ManagedItemVo item);
}
