package gov.va.med.pharmacy.peps.service.common.migration;

import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;

import gov.va.med.pharmacy.peps.common.vo.HazardousWasteImportRow;

/**
 * This class writes the result file for the hazardous waste migration. The result file contains every row from the import file,
 * plus an error flag column to indicate whether a validation error occurred for that row, as well as an error message column
 * describing the error.
 */
public class HazardousWasteMigrationResultFileWriter extends AbstractMigrationResultFileWriter<HazardousWasteImportRow>{
    @Override
    protected XSSFSheet addDetailRow(XSSFSheet sheet, HazardousWasteImportRow importRow, int rowNumber) {
        XSSFRow row = sheet.createRow(rowNumber);
        
        XSSFCell cell = row.createCell(HazardousWasteMigrationConstants.VUID_COLUMN);
        cell.setCellValue(importRow.getVuid());
        cell = row.createCell(HazardousWasteMigrationConstants.EPL_ID_COLUMN);
        cell.setCellValue(importRow.getEplId() == null ? "" : String.valueOf(importRow.getEplId()));
        cell = row.createCell(HazardousWasteMigrationConstants.CMOP_ID_COLUMN);
        cell.setCellValue(importRow.getCmopId());
        cell = row.createCell(HazardousWasteMigrationConstants.NDF_PRODUCT_IEN_COLUMN);
        cell.setCellValue(importRow.getNdfProductIen() == null ? "" : String.valueOf(importRow.getNdfProductIen()));
        cell = row.createCell(HazardousWasteMigrationConstants.VA_PRODUCT_NAME_COLUMN);
        cell.setCellValue(importRow.getVaProductName());
        cell = row.createCell(HazardousWasteMigrationConstants.GENERIC_NAME_COLUMN);
        cell.setCellValue(importRow.getGenericName());
        cell = row.createCell(HazardousWasteMigrationConstants.SCHEDULE_NAME_COLUMN);
        cell.setCellValue(importRow.getScheduleName());
        cell = row.createCell(HazardousWasteMigrationConstants.DOSAGE_FORM_NAME_COLUMN);
        cell.setCellValue(importRow.getDosageFormName());
        cell = row.createCell(HazardousWasteMigrationConstants.HAZARDOUS_TO_HANDLE_COLUMN);
        cell.setCellValue(importRow.getHaz2HandleFlagStr());
        cell = row.createCell(HazardousWasteMigrationConstants.HAZARDOUS_TO_DISPOSE_COLUMN);
        cell.setCellValue(importRow.getHaz2DisposeFlagStr());
        cell = row.createCell(HazardousWasteMigrationConstants.PRIMARY_EPA_COLUMN);
        cell.setCellValue(importRow.getPrimaryEPA());
        cell = row.createCell(HazardousWasteMigrationConstants.WASTE_SORT_CODE_COLUMN);
        cell.setCellValue(importRow.getWasteSortCode());
        cell = row.createCell(HazardousWasteMigrationConstants.DOT_SHIPPING_NAME_COLUMN);
        cell.setCellValue(importRow.getDotShippingName());
        cell = row.createCell(HazardousWasteMigrationConstants.ERROR_FLAG_COLUMN);
        cell.setCellValue(importRow.getErrorFlag());
        cell = row.createCell(HazardousWasteMigrationConstants.ERROR_COLUMN);
        cell.setCellValue(importRow.getErrorMessage());                
        return sheet;
    }

    @Override
    protected XSSFSheet addHeaderRow(XSSFSheet sheet) {
        XSSFRow row = sheet.createRow(0);
        XSSFCell cell = row.createCell(HazardousWasteMigrationConstants.VUID_COLUMN);
        cell.setCellValue("VUID");
        cell = row.createCell(HazardousWasteMigrationConstants.EPL_ID_COLUMN);
        cell.setCellValue("EPL ID");
        cell = row.createCell(HazardousWasteMigrationConstants.CMOP_ID_COLUMN);
        cell.setCellValue("CMOP_ID");
        cell = row.createCell(HazardousWasteMigrationConstants.NDF_PRODUCT_IEN_COLUMN);
        cell.setCellValue("NDF_PRODUCT_IEN");        
        cell = row.createCell(HazardousWasteMigrationConstants.VA_PRODUCT_NAME_COLUMN);
        cell.setCellValue("VA_PRODUCT_NAME");
        cell = row.createCell(HazardousWasteMigrationConstants.GENERIC_NAME_COLUMN);
        cell.setCellValue("GENERIC_NAME");
        cell = row.createCell(HazardousWasteMigrationConstants.SCHEDULE_NAME_COLUMN);
        cell.setCellValue("SCHEDULE_NAME");
        cell = row.createCell(HazardousWasteMigrationConstants.DOSAGE_FORM_NAME_COLUMN);
        cell.setCellValue("DF_NAME");
        cell = row.createCell(HazardousWasteMigrationConstants.HAZARDOUS_TO_HANDLE_COLUMN);
        cell.setCellValue("HAZARDOUS_TO_HANDLE");
        cell = row.createCell(HazardousWasteMigrationConstants.HAZARDOUS_TO_DISPOSE_COLUMN);
        cell.setCellValue("HAZARDOUS_TO_DISPOSE");
        cell = row.createCell(HazardousWasteMigrationConstants.PRIMARY_EPA_COLUMN);
        cell.setCellValue("PRIMARY_EPA");
        cell = row.createCell(HazardousWasteMigrationConstants.WASTE_SORT_CODE_COLUMN);
        cell.setCellValue("WASTE_SORT_CODE");
        cell = row.createCell(HazardousWasteMigrationConstants.DOT_SHIPPING_NAME_COLUMN);
        cell.setCellValue("DOT_SHIPPING_NAME");
        cell = row.createCell(HazardousWasteMigrationConstants.ERROR_FLAG_COLUMN);
        cell.setCellValue("Error Y/N");
        cell = row.createCell(HazardousWasteMigrationConstants.ERROR_COLUMN);
        cell.setCellValue("Error Message");
        return sheet;
    }
}
