package gov.va.med.pharmacy.peps.service.common.session;

import java.io.File;
import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorImportRow;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;
import gov.va.med.pharmacy.peps.service.common.session.impl.FormularyDesignatorMigrationInProgressException;

/**
 * Service layer for Formulary Designator migration.
 */
public interface FormularyDesignatorMigrationService {

    /**
     * Validate and migrate the given rows.
     * 
     * @param rows to process
     * @return rows updated to contain any validation or processing
     * errors.
     */
    List<FormularyDesignatorImportRow> process(
            List<FormularyDesignatorImportRow> rows);

    /**
     * Get the status of the migration, i.e., in progress, done, etc.
     * 
     * @return MigrationStatus object representing the current status
     * of the migration.
     */
    MigrationStatus getStatus();

    /**
     * Get the Excel file containing the results of the migration, including one
     * row per error.
     * 
     * @return Excel file containing any errors which occurred during the migration.
     */
    File getResultFile();

    /**
     * Write the Excel file containing the results of the migration, including one
     * row per error.
     * 
     * @param rows from the migration, to be filtered for inclusion in the error file.
     */
    void writeResultFile(List<FormularyDesignatorImportRow> rows);

    /**
     * Initialize the migration process, in preparation for starting a new migration.
     * 
     * @param rowCount the total number of rows to be processed during this migration
     * @param user who is running the migration
     * @throws FormularyDesignatorMigrationInProgressException if a migration is already
     * in progress, so that it is not possible to start more than one migration at a time.
     */
    void initialize(int rowCount, UserVo user) 
            throws FormularyDesignatorMigrationInProgressException;  
}
