package gov.va.med.pharmacy.peps.service.common.session;

import java.util.List;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;

/**
 * SiteListService.
 * 
 * @author DNS   RUSTRA
 *
 */
public interface SiteListService {

    /**
     * Returns the existing excluded sites.
     * 
     * @return List<ComplianceSiteVo>
     */
    public List<ComplianceSiteVo> getExcludedComplianceSites();

    /**
     * Returns the existing included sites.
     * 
     * @return List<ComplianceSiteVo>
     */
    public List<ComplianceSiteVo> getIncludedComplianceSites();

    /**
     * Saves the excluded sites.
     * 
     * @param excludedSites list of ComplianceSiteVo instances
     * @param user UserVo instance
     */
    public void saveExcludedSites(List<ComplianceSiteVo> excludedSites, UserVo user);

    /**
     * Saves the included sites.
     * 
     * @param includedSites list of ComplianceSiteVo instances
     * @param user UserVo instance
     */
    public void saveIncludedSites(List<ComplianceSiteVo> includedSites, UserVo user);

    /**
     * Returns all the VistaSiteVo instances.
     * 
     * @return List<VistaSiteVo>
     */
    public List<VistaSiteVo> getVistaSites();

    /**
     * Returns the VistaSite instance for the given station number.
     * 
     * @param stationNumber String parameter.
     * @return VistaSiteVo instance.
     */
    public VistaSiteVo getVistaSiteByStationNumber(String stationNumber);

}
