package gov.va.med.pharmacy.peps.service.common.session.impl;

import gov.va.med.pharmacy.peps.common.vo.MigrationRow;
import gov.va.med.pharmacy.peps.common.vo.UserVo;

/**
 * This class processes migration for a single input row of the generic type specified.
 *
 * @param <R> the generic type of MigrationRow processed
 */
public abstract class AbstractMigrator <R extends MigrationRow> {
    
    /**
     * Migrates the given row as the given user.
     * 
     * @param row to migrate
     * @param user performing the migration
     * @return the same row updated to contain any errors which occurred during processing.
     */
    public abstract R migrate(R row, UserVo user);
    
    /**
     * Converts a string to a Boolean.  "Y" = {@code Boolean.TRUE}; anything else = {@code Boolean.FALSE}.
     *
     * @param ynBlankStr String containing "Y", "N", blank, or null.
     * @return the Boolean "Y" = {@code Boolean.TRUE}; anything else = {@code Boolean.FALSE}.
     */
    protected Boolean toBoolean(String ynBlankStr) {
        return Boolean.valueOf("Y".equals(ynBlankStr));
    }
}
