package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.capability.ComplianceSiteDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.SiteListDomainCapability;
import gov.va.med.pharmacy.peps.service.common.session.SiteListService;

/**
 * SiteListService implementor.
 * 
 * @author DNS   RUSTRA
 *
 */
@Service
public class SiteListServiceImpl implements SiteListService {

    /**
     * ComplianceSiteDomainCapability.
     */
    @Autowired
    private ComplianceSiteDomainCapability complianceSiteDomainCapability;

    /**
     * SiteListDomainCapability.
     */
    @Autowired
    private SiteListDomainCapability siteListDomainCapability;

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.service.common.session.SiteListService#getExcludedComplianceSites()
     */
    @Override
    @Transactional(readOnly = true)
    public List<ComplianceSiteVo> getExcludedComplianceSites() {
        return complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.EXCLUDE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.service.common.session.SiteListService#getIncludedComplianceSites()
     */
    @Override
    @Transactional(readOnly = true)
    public List<ComplianceSiteVo> getIncludedComplianceSites() {
        return complianceSiteDomainCapability.retrieveByIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.service.common.session.SiteListService#getVistaSites()
     */
    @Override
    @Transactional(readOnly = true)
    public List<VistaSiteVo> getVistaSites() {
        return (List<VistaSiteVo>) siteListDomainCapability.retrieveAll();
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.service.common.session.SiteListService#getVistaSiteByStationNumber(java.lang.String)
     */
    @Override
    @Transactional
    public VistaSiteVo getVistaSiteByStationNumber(String stationNumber) {
        return siteListDomainCapability.retrieveByStationNumber(stationNumber);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.service.common.session.SiteListService#saveExcludedSites(java.util.List,
     * gov.va.med.pharmacy.peps.common.vo.UserVo)
     */
    @Override
    @Transactional
    public void saveExcludedSites(List<ComplianceSiteVo> excludedSites, UserVo user) {
        for (ComplianceSiteVo complianceSiteVo : excludedSites)
            complianceSiteVo.setIncludeExcludeIndicator(ComplianceSiteVo.EXCLUDE);

        complianceSiteDomainCapability.saveList(excludedSites, ComplianceSiteVo.EXCLUDE, user);
    }

    /*
     * (non-Javadoc)
     * 
     * @see gov.va.med.pharmacy.peps.service.common.session.SiteListService#saveIncludedSites(java.util.List,
     * gov.va.med.pharmacy.peps.common.vo.UserVo)
     */
    @Override
    @Transactional
    public void saveIncludedSites(List<ComplianceSiteVo> includedSites, UserVo user) {
        for (ComplianceSiteVo complianceSiteVo : includedSites)
            complianceSiteVo.setIncludeExcludeIndicator(ComplianceSiteVo.INCLUDE);

        complianceSiteDomainCapability.saveList(includedSites, ComplianceSiteVo.INCLUDE, user);
    }

}
