package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.easymock.EasyMock.*;
import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.utility.test.generator.NdcGenerator;
import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.NdcVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.capability.ManagedItemCapability;

/**
 * Automated tests for {@link ProductRulesCapabilityImpl}.
 */
public class ProductRulesCapabilityImplTest extends EasyMockSupport {
    
    private static final Logger LOG = LogManager.getLogger(ProductRulesCapabilityImplTest.class);
    
    /**
     * Stored as a member variable for convenient creation in {@link #setup()}.
     */
    private UserGenerator userGenerator;

    /**
     * Mock stored as a member variable for convenient creation in {@link #setup()}.
     */
    private ManagedItemCapability mockManagedItemCapability;
    
    @Before
    public void setup() {
        userGenerator = new UserGenerator();
        mockManagedItemCapability = mock(ManagedItemCapability.class);
    }

    @Test
    public void testHandleEnforceRules_NdcNotFound() throws ValidationException {
        
        /* Setup */
        final ProductVo product = new ProductGenerator().getFirst();
        product.setItemStatus(ItemStatus.INACTIVE);
        final NdcVo ndc = new NdcGenerator().getFirst();
        product.setNdcs(Arrays.asList(ndc));
        final UserVo user = userGenerator.getNationalManagerOne();
        
        final ProductRulesCapabilityImpl objUnderTest = new ProductRulesCapabilityImpl();
        objUnderTest.setManagedItemCapability(mockManagedItemCapability);
        
        // Expected method calls.
        expect(
            mockManagedItemCapability.retrieve(ndc.getId(), ndc.getEntityType()))
            .andThrow(new ItemNotFoundException(ItemNotFoundException.ITEM_NOT_FOUND));
        
        replayAll();
        
        /* Behavior & Verification */
        try {
            objUnderTest.handleEnforceRules(product, user, true);
            fail("Expected exception.");
        } catch (RuntimeException ex) {
            LOG.debug("Caught expected exception.", ex);
        }
        
        verifyAll();
    }

    @Test
    public void testHandleEnforceRules_OiNotFound() throws ValidationException {
        
        /* Setup */
        final ProductVo product = new ProductGenerator().getFirst();
        product.setItemStatus(ItemStatus.INACTIVE);
        // Ensure the product has no NDCs: we don't want to check them right now.
        product.setNdcs(new ArrayList<>());
        final UserVo user = userGenerator.getNationalManagerOne();
        
        final ProductRulesCapabilityImpl objUnderTest = new ProductRulesCapabilityImpl();
        objUnderTest.setManagedItemCapability(mockManagedItemCapability);
        
        // Expected method calls.
        expect(
            mockManagedItemCapability.retrieve(product.getOrderableItem().getId(), product.getOrderableItem().getEntityType()))
            .andThrow(new ItemNotFoundException(ItemNotFoundException.ITEM_NOT_FOUND));
        
        replayAll();
        
        /* Behavior & Verification */
        try {
            objUnderTest.handleEnforceRules(product, user, true);
            fail("Expected exception.");
        } catch (RuntimeException ex) {
            LOG.debug("Caught expected exception.", ex);
        }
        
        verifyAll();
    }

}
