package gov.va.med.pharmacy.peps.presentation.common.auth;

/**
 * Enumerates the headers that can come from SSOi. Some of these come on every request, some only on particular
 * requests.
 */
public enum SsoiHeader {

  /**
   * Numeric identifier for the user.
   */
  VAUID,
  
  /**
   * Contains the user's first name.
   */
  FIRSTNAME,
  
  /**
   * Contains the user's last name.
   */
  LASTNAME,

  /**
   * The URL to the SSOi landing page. The application should direct the user to this page after logging out of the
   * application. Accessing this page will <em>not</em> log the user out of SSOi.
   */
  SSOI_LANDING_URL,
  
  /**
   * The URL to the SSOi logout page. Accessing this page <em>will</em> log the user out of SSOi.
   */
  SSOI_LOGGEDOUT_URL;
  
  public String getHeaderName() {
    // Each enum name is identical to the header name, except with underscores instead of dashes.
    return name().replace('_', '-');
  }
}
