package gov.va.med.pharmacy.peps.presentation.common.controller;

import java.util.Locale;

import gov.va.med.pharmacy.peps.common.vo.EntityType;

/**
 * Provides common URLs used in multiple places in PPS. Probably many constants from {@link ControllerConstants} should
 * be moved here.
 */
public class PpsUrls {

    /**
     * The "tip" of the URL for the item edit page. Starts with a slash. The entity type and item ID must be prepended
     * to this string to make the actual URL.
     */
    public static final String PAGE_EDIT = "/edit.go";

    /**
     * The "tip" of the URL for the rematch page. Starts with a slash. The entity type and item ID must be prepended
     * to this string to make the actual URL.
     */
    public static final String PAGE_REMATCH = "/rematch.go";
    
    /**
     * Returns the servlet context-relative URL for the specified item page.
     * 
     * @param entityType the item's entity type
     * @param itemId the item's ID
     * @param page the actual page URL. Must start with a slash.
     * @return the context-relative URL, with <em>no</em> leading slash
     */
    public static String getItemUrl(final EntityType entityType, final String itemId, final String page) {
        return entityType.toLowerCase(Locale.US) + "/" + itemId + page;
    }

    /**
     * Convenience method equivalent to {@link #getItemUrl(EntityType, String, String)} for {@link #PAGE_EDIT}.
     * 
     * @param entityType the item's entity type
     * @param itemId the item's ID
     * @return the context-relative URL, with <em>no</em> leading slash
     */
    public static String getItemEditUrl(final EntityType entityType, final String itemId) {
        return getItemUrl(entityType, itemId, PAGE_EDIT);
    }

}
