/**
 * Source file created in 2011 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.presentation.common.controller.bean;


import java.util.ArrayList;
import java.util.Collection;
import java.util.Objects;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.PartialSaveVo;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.RequestVo;
import gov.va.med.pharmacy.peps.presentation.common.utility.DifferenceUtility;


/**
 * A bean to hold state while editing a ManagedItem.
 */
public class EditManagedItemBean {
    
    private static final Logger LOG = LogManager.getLogger(EditManagedItemBean.class);

    /** item */
    private ManagedItemVo item;

    /** originalItem */
    private ManagedItemVo originalItem;

    /** differences */
    private Collection<ModDifferenceVo> differences;

    /** partialSave */
    private PartialSaveVo partialSave;
    
    /** rematch */
    private RematchVo rematch;

    /** mainRequest */
    private RequestVo mainRequest;
    
    /**
     * 
     * Description here
     *
     */
    public EditManagedItemBean() {
        super();
        
        differences = new ArrayList<>();
    }

    /**
     * 
     * getItem
     *
     * @return the modified item
     */
    public ManagedItemVo getItem() {
        return item;
    }

    /**
     * 
     * setItem
     *
     * @param item the modified ifem
     */
    public void setItem(ManagedItemVo item) {
        this.item = item;
    }

    /**
     * 
     * getOriginalItem
     *
     * @return originalIem
     */
    public ManagedItemVo getOriginalItem() {
        return originalItem;
    }

    /**
     * 
     * setOriginalItem
     *
     * @param originalItem the original item
     */
    public void setOriginalItem(ManagedItemVo originalItem) {
        this.originalItem = originalItem;
    }

    /**
     * Returns the stored collection of {@link ModDifferenceVo}s.
     *
     * @return maybe empty, but never null.
     */
    public Collection<ModDifferenceVo> getDifferences() {
        return differences;
    }

    /**
     * Sets the stored collection of {@link ModDifferenceVo}s.
     *
     * @param differences May be empty, but not null.
     */
    public void setDifferences(Collection<ModDifferenceVo> differences) {
        this.differences = Objects.requireNonNull(differences);
    }

    /**
     * 
     * getPartialSave
     *
     * @return the partial save
     */
    public PartialSaveVo getPartialSave() {
        return partialSave;
    }

    /**
     * 
     * setPartialSave
     *
     * @param partialSave the partial save
     */
    public void setPartialSave(PartialSaveVo partialSave) {
        this.partialSave = partialSave;
    }

    /**
     * Returns the associated rematch.
     * @return possibly null.
     */
    public RematchVo getRematch() {
      return rematch;
    }

    public void setRematch(RematchVo rematch) {
      this.rematch = rematch;
    }

    /**
     * getMainRequest
     * @return the mainRequest
     */
    public RequestVo getMainRequest() {
        return mainRequest;
    }

    /**
     * setMainRequest
     * @param mainRequest the mainRequest to set
     */
    public void setMainRequest(RequestVo mainRequest) {
        this.mainRequest = mainRequest;
    }
    
    /**
     * Resets the item to the original state (i.e., {@link #getOriginalItem()}). The differences will be cleared
     * accordingly.
     */
    public void resetItem() {
        LOG.debug("Resetting item to original.");
        item = originalItem.copy();
        rematch = null;
        differences = new ArrayList<>();
    }
    
    /**
     * Returns whether triggering a manual rematch is currently allowed in this edit session.
     * @return true if allowed, false otherwise
     */
    public boolean canManualRematch() {
        return item.canRematch()
            && mainRequest == null; // Don't permit manual rematches while a request is in progress.
    }
    
    /**
     * Convenience method: equivalent to calling {@link DifferenceUtility#hasEditableDifference(Collection)} on 
     * {@link #getDifferences()}.
     * @return the return value from {@link DifferenceUtility#hasEditableDifference(Collection)}
     */
    public boolean hasEditableDifference() {
        return DifferenceUtility.hasEditableDifference(getDifferences());
    }

}
