

package gov.va.med.pharmacy.peps.presentation.common.controller.migration;


import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.presentation.common.controller.PepsController;
import gov.va.med.pharmacy.peps.presentation.common.spring.annotation.RoleNeeded;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;
import gov.va.med.pharmacy.peps.service.common.session.Ppsn3MigrationService;

import java.io.File;
import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

/**
 * Controller for Drug Drug Interaction Migration.
 */
@Controller
@RoleNeeded(roles = { Role.PSS_PPSN_MIGRATOR })
public class MigrationDdiController extends PepsController {

    private static final String PAGE = "migration-ddi-page";
    
    private static final Logger LOG = LogManager.getLogger(MigrationDdiController.class);

    @Autowired
    private Ppsn3MigrationService ppsn3MigrationService;

    /**
     * Gets the Migrate DDI page.
     *
     * @return the page
     */
    @RequestMapping(value = "/migrationDDI.go", method = RequestMethod.GET)
    public ModelAndView getPage() {
        pageTrail.clearTrail();
        pageTrail.addPage("migrationDDI", "Migrate DDI Data", true);
        return new ModelAndView(PAGE);
    }

    /**
     * Accepts the input file upload and performs the migration of DDI.
     *
     * @param request the HttpServletRequest
     * @param response the HttpServletResponse
     * @param file the file containing the data to migrate.
     * @return the Spring MVC ModelAndView
     * @throws IOException Signals that an I/O exception has occurred.
     */
    @RequestMapping(value = "/migrationDDI.go", method = RequestMethod.POST)
    public ModelAndView migrateDDI(HttpServletRequest request, HttpServletResponse response,
        @RequestParam("file") MultipartFile file) throws IOException {
        
        File tmp = File.createTempFile("tmp", "txt");
        file.transferTo(tmp);
        LOG.debug("Wrote the file to {}.", tmp);

        ppsn3MigrationService.migrateDdis(getUser(), tmp);

        ModelAndView view = new ModelAndView(PAGE);
        // It would be nice to display a whole MigrationStatus object with counts and everything, but no time for that
        // now.
        view.addObject("status", MigrationStatus.DONE);
        return view;
    }

}
