package gov.va.med.pharmacy.peps.presentation.common.controller.requests;

import java.util.Locale;
import java.util.Map;

import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.presentation.common.utility.JspTagUtility;

/**
 * Support rematch views for MangerRequestsController. 
 */
public class RematchApprovalItem {
    /** Field name prefix to make control name unique. */
    public static final String MATCH_APPROVAL_MASK = "match_approval_";
    private String matchId = null;
    private String vaProductName = null;
    private String cmopId = null;
    private Boolean isAutomaticRematch = null;
    private RequestItemStatus approval = null;
    
    /**
     * Constructor with parameters.
     * @param id String 
     * @param vaProductName String
     * @param cmopId String
     * @param isAuto Boolean
     * @param approval RequestItemStatus
     */
    public RematchApprovalItem(String id, String vaProductName, String cmopId, Boolean isAuto, RequestItemStatus approval){
        this.matchId = id;
        this.vaProductName = vaProductName;
        this.cmopId = cmopId;
        this.isAutomaticRematch = isAuto;
        this.approval = approval;
    }

    /**
     * get match id.
     * @return the matchId
     */
    public String getMatchId() {
        return matchId;
    }

    /**
     * set match id.
     * @param matchId the matchId to set
     */
    public void setMatchId(String matchId) {
        this.matchId = matchId;
    }

    /**
     * get va product name.
     * @return the vaProductName
     */
    public String getVaProductName() {
        return vaProductName;
    }

    /**
     * setter method.
     * @param vaProductName the vaProductName to set
     */
    public void setVaProductName(String vaProductName) {
        this.vaProductName = vaProductName;
    }

    /**
     * getter method.
     * @return the cmopId
     */
    public String getCmopId() {
        return cmopId;
    }

    /**
     * setter method.
     * @param cmopId the cmopId to set
     */
    public void setCmopId(String cmopId) {
        this.cmopId = cmopId;
    }

    /**
     * getter method.
     * @return the isAutomaticRematch
     */
    public Boolean getIsAutomaticRematch() {
        return isAutomaticRematch;
    }

    /**
     * setter method.
     * @param isAutomaticRematch the isAutomaticRematch to set
     */
    public void setIsAutomaticRematch(Boolean isAutomaticRematch) {
        this.isAutomaticRematch = isAutomaticRematch;
    }

    /**
     * getter method.
     * @return the approval
     */
    public Map<String, Enum> getApprovals() {
        return JspTagUtility.getEnumValues(Locale.getDefault(), RequestItemStatus.class);
    }

    /**
     * getter method.
     * @return the approval
     */
    public RequestItemStatus getApproval() {
        return approval;
    }

    /**
     * setter method.
     * @param approval the approval to set
     */
    public void setApproval(RequestItemStatus approval) {
        this.approval = approval;
    }

    /**
     * get approval name as string.
     * @return the approvalName
     */
    public String getApprovalName() {
        return approval.name(); 
    }

}
