/**
 * Source file created in 2011 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.presentation.common.controller;


import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import gov.va.med.pharmacy.peps.common.exception.InterfaceException;
import gov.va.med.pharmacy.peps.common.vo.SiteConfigVo;
import gov.va.med.pharmacy.peps.common.vo.printtemplate.DefaultPrintTemplateFactory;
import gov.va.med.pharmacy.peps.common.vo.printtemplate.PrintTemplateVo;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.session.QuickActionService;
import gov.va.med.pharmacy.peps.service.national.session.ConsoleService;


/**
 * <p>
 * Manage Application Tab.
 * </p>
 * 
 * <p>
 * This controller may be used in PPS-N, but only if PPS-L is deployed. So it is not included in the PP-N build for now.
 * </p>
 */
@Controller
public class ManageApplicationController extends PepsController {

    private static final String NATIONAL_STATUS = "nationalStatus";
    private static final String LOCAL_STATUS = "localStatus";

    private static final String URL_SYSTEM_ACTIONS = "/systemActions.go";
    private static final String URL_CONSOLE = "/console.go";
    private static final String RET_SYS_ACT_EDIT = "system.actions.edit";

    private static final PrintTemplateVo CONSOLE_STATUS_PRINT_TEMPLATE =
        DefaultPrintTemplateFactory.defaultStatusConfiguration();

    /**
     * The tab key for the "Send Drug File" tab.
     */
    public static final String SEND_DRUG_FILE = "send_drug_file";

    @Autowired
    private ConsoleService consoleService;

    @Autowired
    private QuickActionService quickActionService;

    /**
     * System Information GET
     * @param model Spring Model
     * @return console view
     */
    @RequestMapping(value = URL_CONSOLE, method = RequestMethod.GET)
    public String displayConsoleInformation(Model model) {
        pageTrail.clearTrail();
        pageTrail.addPage("viewSystemInfo", "View System Information", true);

        model.addAttribute(NATIONAL_STATUS, retrieveNationalConsoleStatus());
        model.addAttribute(LOCAL_STATUS, retrieveLocalConsoleStatus());
        model.addAttribute("printTemplate", CONSOLE_STATUS_PRINT_TEMPLATE);

        // Note: at time of writing, this view does not exist.
        return "console";
    }

    /**
     * System Actions GET
     * @param tabKey 
     * @param model 
     * @return system.actions.edit view
     */
    @RequestMapping(value = URL_SYSTEM_ACTIONS, method = RequestMethod.GET)
    public String getSystemActions(
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false, defaultValue = SEND_DRUG_FILE) String tabKey,
        Model model) {

        model.addAttribute(ControllerConstants.TAB_KEY, tabKey);

        return RET_SYS_ACT_EDIT;
    }

    /**
     * System Actions POST
     * @param tabKey 
     * @param model 
     * @return system.actions.edit view
     */
    @RequestMapping(value = URL_SYSTEM_ACTIONS, method = RequestMethod.POST)
    public String postSystemActions(
        @RequestParam(value = ControllerConstants.TAB_KEY, required = false, defaultValue = SEND_DRUG_FILE) String tabKey,
        Model model) {
        String status = sendEntireDrugFile();
        model.addAttribute("status", status);
        model.addAttribute(ControllerConstants.TAB_KEY, tabKey);

        return RET_SYS_ACT_EDIT;
    }

    /**
     * 
     * Retrieves the console data (System Information) for national sites
     *
     * @return list of national statuses 
     */
    private List<SiteConfigVo> retrieveNationalConsoleStatus() {

        // pull out the National site config vo and make it a list for the national display table
        ArrayList<SiteConfigVo> tmpNationalSiteConfigList = new ArrayList<SiteConfigVo>();
        tmpNationalSiteConfigList.add(consoleService.getConsole().getNationalSiteConfigInfo());

        return tmpNationalSiteConfigList;
    }

    /**
     * Retrieves the console data (System Information) for all local sites
     * 
     * @return list of local statuses
     */
    private List<SiteConfigVo> retrieveLocalConsoleStatus() {
        return consoleService.getConsole().getLocalSiteConfigInfoList();
    }

    /**
     * Send drug file to Vista.
     * 
     * @return SUCCESS
     */
    private String sendEntireDrugFile() {

        String fileSent;

        try {
            quickActionService.sendDrugFileToExternalInterface(getUser());
            fileSent = "Request to send entire drug file was successful";
        } catch (InterfaceException e) {

            // Note: Catching a generic interface exception is generally a bad idea, but in this case we need to catch
            // anything coming from the interface and display a message indicating a problem

            fileSent = "There was an issue sending the drug file. Please check the logs for details.";
        }

        return fileSent;
    };

}
