package gov.va.med.pharmacy.peps.presentation.common.auth;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import org.junit.Test;
import org.springframework.mock.web.MockHttpServletRequest;

/**
 * Unit tests for {@link PpsAuthentication}.
 */
public class PpsAuthenticationTest {

    @Test
    public void testGetLogoutTargetUrl_ValidLanding() {
        
        /* Setup */
        final String landingUrl = "https://foo.DNS   /landing.aspx";
        final String logoutUrl = "https://foo.DNS   /logout.aspx";

        MockHttpServletRequest request = new MockHttpServletRequest();
        request.addHeader(SsoiHeader.SSOI_LANDING_URL.getHeaderName(), landingUrl);
        request.addHeader(SsoiHeader.SSOI_LOGGEDOUT_URL.getHeaderName(), logoutUrl);
        
        /* Behavior & Verification */
        assertThat(PpsAuthentication.getLogoutTargetUrl(request), equalTo(landingUrl));
    }

    @Test
    public void testGetLogoutTargetUrl_InvalidLanding() {
        
        /* Setup */
        final String landingUrl = "https://www.google.com/landing.aspx";
        final String logoutUrl = "https://foo.DNS   /logout.aspx";

        MockHttpServletRequest request = new MockHttpServletRequest();
        request.addHeader(SsoiHeader.SSOI_LANDING_URL.getHeaderName(), landingUrl);
        request.addHeader(SsoiHeader.SSOI_LOGGEDOUT_URL.getHeaderName(), logoutUrl);
        
        /* Behavior & Verification */
        assertThat(PpsAuthentication.getLogoutTargetUrl(request), equalTo(logoutUrl));
    }

    @Test
    public void testGetLogoutTargetUrl_NoLanding() {
        
        /* Setup */
        final String logoutUrl = "https://foo.DNS   /logout.aspx";

        MockHttpServletRequest request = new MockHttpServletRequest();
        request.addHeader(SsoiHeader.SSOI_LOGGEDOUT_URL.getHeaderName(), logoutUrl);
        
        /* Behavior & Verification */
        assertThat(PpsAuthentication.getLogoutTargetUrl(request), equalTo(logoutUrl));
    }

    @Test
    public void testGetLogoutTargetUrl_Default() {
        /* Setup */
        MockHttpServletRequest request = new MockHttpServletRequest();
        
        /* Behavior & Verification */
        assertNull(PpsAuthentication.getLogoutTargetUrl(request));
    }

}
